""" renderers """
from pyramid.response import Response
from pyramid.httpexceptions import HTTPForbidden
from pyramid.interfaces import IAuthorizationPolicy
from pyramid.interfaces import IAuthenticationPolicy
from pyramid.threadlocal import get_current_registry


checkPermission = None

def set_checkpermission(func):
    global checkPermission
    checkPermission = func
    return func


@set_checkpermission
def default_checkpermission(permission, context, request=None, throw=False):
    AUTH = get_current_registry().queryUtility(IAuthenticationPolicy)
    if AUTH:
        AUTHZ = get_current_registry().queryUtility(IAuthorizationPolicy)
        if AUTHZ:
            principals = AUTH.effective_principals(request)
            return AUTHZ.permits(context, principals, permission)

    return True


class PermissionRenderer(object):

    def __init__(self, permission, factory):
        if callable(permission):
            def permChecker(context, request):
                if not permission(context, request):
                    msg = getattr(
                        request, 'authdebug_message',
                        'Unauthorized: %s failed permission check'%factory)
                    raise HTTPForbidden(msg)
        else:
            def permChecker(context, request):
                global checkPermission
                if not checkPermission(permission, context, request):
                    msg = getattr(
                        request, 'authdebug_message',
                        'Unauthorized: %s failed permission check'%factory)
                    raise HTTPForbidden(msg)
        self.checker = permChecker

    def __call__(self, context, request, content):
        self.checker(context, request)
        return content


class ViewRenderer(object):

    def __init__(self, factory, content_type='text/html'):
        self.factory = factory
        self.content_type = content_type

    def __call__(self, context, request, content):
        request.response.content_type = self.content_type
        return self.factory(context, request)[1]


class TemplateRenderer(ViewRenderer):

    def __init__(self, factory, template, content_type='text/html'):
        self.factory = factory
        self.template = template
        self.content_type = content_type

    def __call__(self, context, request, content):
        view, result = self.factory(context, request)

        if isinstance(result, Response):
            return result

        kwargs = {'view': view,
                  'context': context,
                  'request': request}
        if type(result) is dict:
            kwargs.update(result)

        request.response.content_type = self.content_type
        return self.template(**kwargs)



""" Source code view """
import os.path
import pkg_resources
from ptah import view
from pyramid.httpexceptions import HTTPFound

from manage import PtahManageRoute


class SourceView(view.View):
    view.pview(
        'source.html', PtahManageRoute,
        template = view.template('ptah.manage:templates/source.pt'))

    __doc__ = 'Source introspection page.'
    __intr_path__ = '/ptah-manage/source.html'

    source = None
    format = None

    def update(self):
        name = self.request.params.get('pkg')
        if name is None:
            raise HTTPFound(location='.')

        dist = None
        pkg_name = name
        while 1:
            try:
                dist = pkg_resources.get_distribution(pkg_name)
                if dist is not None:
                    break
            except pkg_resources.DistributionNotFound:
                if '.' not in pkg_name:
                    break
                pkg_name = pkg_name.rsplit('.',1)[0]

        if dist is None:
            self.source = None

        names = name[len(pkg_name)+1:].split('.')
        path = '%s.py'%os.path.join(*names)
        try:
            abspath = pkg_resources.resource_filename(pkg_name, path)
        except:
            raise HTTPFound(location='.')

        if os.path.isfile(abspath):
            self.file = abspath
            self.name = '%s.py'%names[-1]
            self.pkg_name = pkg_name
            source = open(abspath, 'rb').read()

            if not self.format:
                from pygments import highlight
                from pygments.lexers import PythonLexer
                from pygments.formatters import HtmlFormatter

                html = HtmlFormatter(
                    linenos='inline',
                    lineanchors='sl',
                    anchorlinenos=True,
                    noclasses = True,
                    cssclass="ptah-source")

                def format(self, code, highlight=highlight,
                           lexer = PythonLexer()):
                    return highlight(code, lexer, html)

                self.__class__.format = format

            self.source = self.format(source)



""" settings """
import colander
import ConfigParser
import logging
import os.path

from zope import interface
from zope.interface.interface import InterfaceClass

import api
import schema
from directives import event, subscriber, DirectiveInfo, Action

log = logging.getLogger('ptah.config')


class SettingsInitializing(object):
    """ Settings initializing event """
    event('Settings initializing event')

    config = None
    registry = None

    def __init__(self, config, registry):
        self.config = config
        self.registry = registry


class SettingsInitialized(object):
    """ ptah sends this event when settings initialization is completed. """
    event('Settings initialized event')

    config = None
    registry = None

    def __init__(self, config, registry):
        self.config = config
        self.registry = registry


_marker = object()

SETTINGS_ID = 'settings'
SETTINGS_OB_ID = 'ptah.config:settings'
SETTINGS_GROUP_ID = 'ptah.config:settings-group'


def get_settings():
    return api.get_cfg_storage(SETTINGS_OB_ID)


@subscriber(api.Initialized)
def init_settings(ev):
    settings = Settings()
    ev.registry.storage[SETTINGS_OB_ID] = settings

    # complete settings initialization
    for grp in ev.registry.storage[SETTINGS_GROUP_ID].values():
        settings.register(grp)


def initialize_settings(cfg, pconfig, section=ConfigParser.DEFAULTSECT):
    settings = pconfig.registry.storage[SETTINGS_OB_ID]
    if settings.initialized:
        raise RuntimeError(
            "initialize_settings has been called more than once.")

    log.info('Initializing ptah settings')

    settings.config = pconfig
    settings.initialized = True

    here = cfg.get('here', './')

    include = cfg.get('include', '')
    for f in include.split('\n'):
        f = f.strip()
        if f and os.path.exists(f):
            parser = ConfigParser.SafeConfigParser()
            parser.read(f)
            if section == ConfigParser.DEFAULTSECT or \
                    parser.has_section(section):
                cfg.update(parser.items(section, vars={'here': here}))

    pconfig.begin()
    try:
        settings.init(cfg)
        api.notify(SettingsInitializing(pconfig, pconfig.registry))
        api.notify(SettingsInitialized(pconfig, pconfig.registry))
    except Exception, e:
        raise api.StopException(e)
    finally:
        pconfig.end()


def register_settings(name, *nodes, **kw):
    title = kw.get('title', '')
    description = kw.get('description', '')
    validator = kw.get('validator', None)

    iname = name
    for ch in ('.', '-'):
        iname = iname.replace(ch, '_')

    category = InterfaceClass(
        'SettingsGroup:%s' % iname.upper(), (),
        __doc__='Settings group: %s' % name,
        __module__='ptah.config.settings')

    group = Group(name, title, description, category, *nodes)

    if validator is not None:
        if type(validator) not in (list, tuple):
            validator = validator,

        for v in validator:
            group.schema.validator.add(v)

    ac = Action(
        lambda config, group: config.get_cfg_storage(SETTINGS_GROUP_ID)\
            .update({group.name: group}),
        (group,),
        discriminator=(SETTINGS_GROUP_ID, name))

    info = DirectiveInfo()
    info.attach(ac)

    return group


class Settings(dict):
    """ settings management system """

    config = None
    initialized = False

    def __init__(self):
        self.schema = schema.SchemaNode(schema.Mapping())

    def _load(self, rawdata, setdefaults=False):
        try:
            rawdata = dict((k.lower(), v) for k, v in rawdata.items())
            data = self.schema.unflatten(rawdata)
        except Exception, exc:
            log.error('Error loading settings: %s', exc)
            if setdefaults:
                raise
            return

        try:
            data = self.schema.deserialize(data)
        except colander.Invalid, e:
            errs = e.asdict()
            if setdefaults:
                log.error('Error loading default settings: \n%s' % (
                        '\n'.join('%s: %s' % (k, v) for k, v in errs.items())))
                return

            log.error('Error loading settings, reloading with defaults: \n%s' %
                    ('\n'.join('%s: %s' % (k, v) for k, v in errs.items())))

            defaults = self.schema.flatten(self.schema.serialize())

            for k in errs.keys():
                rawdata.pop(k)
                rawdata[k] = defaults[k]

            data = self.schema.deserialize(self.schema.unflatten(rawdata))

        for name, group in self.items():
            if name in data and data[name]:
                if setdefaults:
                    for k, v in data[name].items():
                        if v is not colander.null:
                            group.schema[k].default = v

                group.update(data[name])

    def init(self, defaults=None):
        for group in self.values():
            data = {}
            for node in group.schema.children:
                node.default = node._origin_default
                if node.default is not colander.null:
                    data[node.name] = node.default

            group.update(data)

        if defaults:
            self._load(defaults, True)

    def export(self, default=False):
        result = {}
        for name, group in self.items():
            data = dict((node.name, node.default) for node in group.schema)
            data.update(group.items())
            result[name] = data

        result = self.schema.serialize(result)

        if not default:
            for prefix, group in self.items():
                schema = group.schema
                for key, val in group.items():
                    if val == schema[key].default:
                        del result[prefix][key]

        return self.schema.flatten(result)

    def register(self, group):
        if group.name not in self:
            self.schema.add(group.schema)
            self[group.name] = group


class GroupValidator(object):

    def __init__(self):
        self._validators = []

    def add(self, validator):
        if validator not in self._validators:
            self._validators.append(validator)

    def __call__(self, node, appstruct):
        error = None
        for validator in self._validators:
            try:
                validator(node, appstruct)
            except colander.Invalid, e:
                if error is None:
                    error = colander.Invalid(node)

                if e.node is node:
                    error.add(e)
                else:
                    error.add(e, node.children.index(e.node))

        if error is not None:
            raise error


class Group(object):

    def __init__(self, name, title, description, category, *nodes):
        self.name = name
        self.title = title
        self.description = description
        self.category = category
        self.schema = schema.SchemaNode(
            schema.Mapping(),
            name=name,
            required=False,
            validator=GroupValidator())

        interface.directlyProvides(self, category)

        for node in nodes:
            if not isinstance(node, schema.SchemaNode):
                raise RuntimeError(
                    "Node '%s' has to be instance of "
                    "ptah.config.SchemaNode" % node.name)

            self.schema.add(node)

    def get(self, name, default=None):
        try:
            data = api.get_cfg_storage(SETTINGS_ID)[self.name]
            if name in data:
                return data[name]
        except (KeyError, AttributeError):
            pass

        if name in self.schema:
            return self.schema[name].default

        return default

    def keys(self):
        return [node.name for node in self.schema]

    def items(self):
        return [(key, self.get(key)) for key in self.keys()]

    def update(self, data):
        api.get_cfg_storage(SETTINGS_ID)[self.name].update(data)

    def __getattr__(self, attr, default=_marker):
        res = self.get(attr, default)
        if res is _marker:
            raise AttributeError(attr)
        return res

    def __getitem__(self, name):
        res = self.get(name, _marker)
        if res is _marker:
            raise KeyError(name)
        return res

    def __setitem__(self, name, value):
        data = api.get_cfg_storage(SETTINGS_ID)
        try:
            data[self.name][name] = value
        except KeyError:
            data[self.name] = {}
            data[self.name][name] = value



import ptah
from pyramid.testing import DummyRequest

from base import Base


class TestUriView(Base):

    def setUp(self):
        self._setup_pyramid()

    def test_uri_view(self):
        from ptah.manage.uri import UriResolver
        self._init_ptah()

        request = DummyRequest(
            GET = {'uri': 'ptah-auth:superuser'})

        view = UriResolver(None, request)
        view.update()

        self.assertEqual(view.data[0]['name'],
                         'ptah.authentication.superuser_resolver')

    def test_uri_handler(self):
        from ptah.manage.uri import UriResolver
        self._init_ptah()

        request = DummyRequest(
            POST = {'form.buttons.show': 'Show'})

        view = UriResolver(None, request)
        view.update()

        request = DummyRequest(
            POST = {'form.buttons.show': 'Show', 'uri': 'ptah-auth:superuser'})

        view = UriResolver(None, request)
        view.update()

        self.assertEqual(view.data[0]['name'],
                         'ptah.authentication.superuser_resolver')


@ptah.resolver('uri-intro-test')
def resolver(uri): # pragma: no cover
    return 'Resolved'


class TestUriIntrospect(Base):

    def test_uri_introspect(self):
        from ptah.config import directives
        from ptah.manage.uri import RESOLVER_ID, UriIntrospection

        data = directives.scan(self.__class__.__module__, set())

        actions = []
        for action in data:
            if action.discriminator[0] == RESOLVER_ID:
                actions.append(action)

        ti = UriIntrospection(self.request)
        res = ti.renderActions(*actions)

        self.assertIn('uri-intro-test', res)
        self.assertIn('ptah.manage.tests.test_uri', res)



""" app management module """
import ptah
from ptah import view, form, manage, cms
from pyramid.httpexceptions import HTTPFound


class ApplicationsModule(manage.PtahModule):
    __doc__ = u'A listing of all registered Ptah Applications.'

    title = 'Applications'
    manage.module('apps')

    def __getitem__(self, key):
        for id, factory in cms.get_app_factories().items():
            if factory.name == key:
                return AppFactory(factory, self, self.request)

        raise KeyError(key)


class ApplicationsModuleView(view.View):
    view.pview(
        context=ApplicationsModule,
        template=view.template('ptah.manage:templates/apps.pt'))

    def update(self):
        factories = []
        for factory in cms.get_app_factories().values():
            factories.append((factory.title, factory))

        factories.sort()
        self.factories = [f for _t, f in factories]


class AppFactory(object):

    def __init__(self, factory, context, request):
        self.__name__ = factory.id
        self.__parent__ = context
        self.request = request
        self.factory = factory
        self.app = factory(request)


class SharingForm(form.Form):
    view.pview(
        context = AppFactory,
        template = view.template('ptah.manage:templates/apps-sharing.pt'))

    csrf = True
    fields = form.Fieldset(
        form.FieldFactory(
            'text',
            'term',
            title = u'Search term',
            description = 'Searches users by login and email',
            missing = u'',
            default = u'')
        )


    users = None
    bsize = 15

    def form_content(self):
        return {'term': self.request.session.get('apps-sharing-term', '')}

    def get_principal(self, id):
        return ptah.resolve(id)

    def update(self):
        super(SharingForm, self).update()

        request = self.request
        context = self.context.app

        self.roles = [r for r in ptah.get_roles().values() if not r.system]
        self.local_roles = local_roles = context.__local_roles__

        term = request.session.get('apps-sharing-term', '')
        if term:
            self.users = list(ptah.search_principals(term))

        if 'form.buttons.save' in request.POST:
            users = []
            userdata = {}
            for attr, val in request.POST.items():
                if attr.startswith('user-'):
                    userId, roleId = attr[5:].rsplit('-',1)
                    data = userdata.setdefault(str(userId), [])
                    data.append(str(roleId))
                if attr.startswith('userid-'):
                    users.append(str(attr.split('userid-')[-1]))

            for uid in users:
                if userdata.get(uid):
                    local_roles[str(uid)] = userdata[uid]
                elif uid in local_roles:
                    del local_roles[uid]

            context.__local_roles__ = local_roles

    @form.button('Search', actype=form.AC_PRIMARY)
    def search(self):
        data, error = self.extract()

        if not data['term']:
            self.message('Please specify search term', 'warning')
            return

        self.request.session['apps-sharing-term'] = data['term']
        raise HTTPFound(location = self.request.url)



""" role """
from ptah import config

from base import Base


class Principal(object):

    def __init__(self, uri, name, login):
        self.uri = uri
        self.name = name
        self.login = login


class TestAuthentication(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestAuthentication, self).tearDown()

    def test_auth_simple(self):
        import ptah

        info = ptah.authService.authenticate(
            {'login': 'user', 'password': '12345'})

        self.assertFalse(info.status)

        class Provider(object):
            def authenticate(self, creds):
                if creds['login'] == 'user':
                    return Principal('1', 'user', 'user')

        ptah.register_auth_provider('test-provider', Provider())
        self._init_ptah()

        info = ptah.authService.authenticate(
            {'login': 'user', 'password': '12345'})

        self.assertTrue(info.status)
        self.assertEqual(info.uri, '1')

    def test_auth_checker_default(self):
        import ptah
        self._init_ptah()

        principal = Principal('1', 'user', 'user')

        info = ptah.authService.authenticate_principal(principal)
        self.assertTrue(info.status)
        self.assertEqual(info.uri, '1')
        self.assertEqual(info.message, '')
        self.assertEqual(info.arguments, {})

    def test_auth_checker(self):
        import ptah

        principal = Principal('1', 'user', 'user')

        class Provider(object):
            def authenticate(self, creds):
                if creds['login'] == 'user':
                    return Principal('1', 'user', 'user')

        ptah.register_auth_provider('test-provider', Provider())

        @ptah.auth_checker
        def checker(info):
            info.message = 'Suspended'
            info.arguments['additional'] = 'test'
            return False

        self._init_ptah()

        info = ptah.authService.authenticate(
            {'login': 'user', 'password': '12345'})

        self.assertFalse(info.status)
        self.assertEqual(info.uri, '1')
        self.assertEqual(info.message, 'Suspended')
        self.assertEqual(info.arguments, {'additional': 'test'})

        principal = Principal('1', 'user', 'user')

        info = ptah.authService.authenticate_principal(principal)
        self.assertFalse(info.status)
        self.assertEqual(info.uri, '1')
        self.assertEqual(info.message, 'Suspended')
        self.assertEqual(info.arguments, {'additional': 'test'})

    def test_auth_get_set_userid(self):
        import ptah
        import ptah.util

        self.assertEqual(ptah.authService.get_userid(), None)

        ptah.authService.set_userid('user')
        self.assertEqual(ptah.authService.get_userid(), 'user')

        ptah.util.resetThreadLocalData(None)
        self.assertEqual(ptah.authService.get_userid(), None)

    def test_auth_principal(self):
        import ptah

        principal = Principal('1', 'user', 'user')
        def resolver(uri):
            if uri == 'test:1':
                return principal

        ptah.register_uri_resolver('test', resolver)
        self._init_ptah()

        self.assertEqual(ptah.authService.get_current_principal(), None)

        ptah.authService.set_userid('test:1')
        self.assertEqual(ptah.authService.get_current_principal(), principal)

    def test_auth_principal_login(self):
        import ptah

        principal = Principal('1', 'user', 'user')
        class Provider(object):
            def get_principal_bylogin(self, login):
                if login == 'user':
                    return principal

        ptah.register_auth_provider('test-provider', Provider())
        self._init_ptah()

        self.assertEqual(
            ptah.authService.get_principal_bylogin('user2'), None)

        self.assertEqual(
            ptah.authService.get_principal_bylogin('user'), principal)


class TestPrincipalSearcher(Base):

    def test_principal_searcher(self):
        import ptah

        principal = Principal('1', 'user', 'user')
        def search(term=''):
            if term == 'user':
                yield principal

        ptah.register_principal_searcher('test-provider', search)
        self._init_ptah()

        self.assertEqual(list(ptah.search_principals('user')), [principal])


class TestSuperUser(Base):

    def test_superuser_resolver(self):
        import ptah
        from ptah.authentication import SUPERUSER
        self._init_ptah()

        user = ptah.resolve(ptah.SUPERUSER_URI)
        self.assertIs(user, SUPERUSER)
        self.assertIsNone(ptah.resolve('ptah-auth:unknown'))
        self.assertEqual(repr(user), '<ptah Superuser>')



import ptah
from pyramid.testing import DummyRequest

from base import Base


class TestSettingsModule(Base):

    def test_settings_module(self):
        self._init_ptah()

        from ptah.manage.manage import CONFIG, PtahManageRoute
        from ptah.manage.settings import SettingsModule

        request = DummyRequest()

        ptah.authService.set_userid('test')
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['settings']

        self.assertIsInstance(mod, SettingsModule)

    def test_settings_view(self):
        from ptah.manage.settings import SettingsModule, SettingsView

        request = DummyRequest()

        mod = SettingsModule(None, request)

        res = SettingsView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')



""" tmpl  """
import unittest
from ptah import view, config
from ptah.view import tmpl as tapi


class TestTmplCommand(unittest.TestCase):

    def tearDown(self):
        config.cleanup_system()

    def test_tmpl_err1(self):
        self.assertRaises(
            ValueError,
            view.template, 'unkown.pt')

    def test_tmpl_relative(self):
        path, pkg = tapi.path('templates/msg-error.pt')
        self.assertEqual(path, None)
        self.assertEqual(pkg, 'ptah.view.tests')

    def test_tmpl(self):
        tmpl = view.template('ptah.view.tests:templates/test.pt')
        self.assertTrue('PageTemplateFile' in repr(tmpl))

        tmpl = view.template('ptah.view.tests:templates/test.txt')
        self.assertTrue('PageTextTemplateFile' in repr(tmpl))

        self.assertEqual(tapi.registry['ptah.view.tests'].keys(),
                         ['test.pt', 'test.txt'])

    def test_tmpl_multiple_decl(self):
        tmpl = view.template('ptah.view.tests:templates/test.pt')

        tmpl1 = view.template('ptah.view.tests:templates/test.pt')

        self.assertIs(tmpl, tmpl1)

    def test_tmpl_multiple_disable_packages(self):
        view.template('ptah.view.tests:templates/test.pt', nolayer = True)
        view.template('ptah.view.tests:templates/test.pt', nolayer = True)
        self.assertEqual(
            tapi.registry.keys(), [])

    def test_tmpl_multiple_layer_name(self):
        view.template('ptah.view.tests:templates/test.pt', layer = 'test')
        view.template('ptah.view.tests:templates/test.pt')

        self.assertEqual(tapi.registry.keys(), ['test', 'ptah.view.tests'])

    def test_tmpl_multiple_extra_params(self):
        view.template('ptah.view.tests:templates/test.pt',
                      title = 'Test template',
                      description = 'Test template description')

        self.assertEqual(tapi.registry['ptah.view.tests']['test.pt'][1:3],
                         ['Test template', 'Test template description'])



import transaction
from pyramid.httpexceptions import HTTPNotFound, HTTPForbidden

import ptah
from base import Base


class Content(ptah.cms.Content):

    __type__ = ptah.cms.Type('content', 'Test Content')
    __uri_factory__ = ptah.UriFactory('cms-content')


class Container(ptah.cms.Container):

    __type__ = ptah.cms.Type('container', 'Test Container')
    __uri_factory__ = ptah.UriFactory('cms-container')


class TestLoadApi(Base):
    """ fixme: redesign tests to use custom resolver """

    def test_loadapi_load(self):
        content = Content(title='Content')
        uri = content.__uri__

        ptah.cms.Session.add(content)
        transaction.commit()

        content = ptah.cms.load(uri)
        self.assertEqual(content.__uri__, uri)

    def test_loadapi_load_notfound(self):
        self.assertRaises(HTTPNotFound, ptah.cms.load, 'unknown')

    def test_loadapi_load_with_parents(self):
        content = Content(title='Content')
        container = Container(__name__='container', __path__='/container/')

        c_uri = content.__uri__
        co_uri = container.__uri__
        ptah.cms.Session.add(container)
        ptah.cms.Session.add(content)
        transaction.commit()

        container = ptah.resolve(co_uri)
        container['content'] = ptah.resolve(c_uri)
        transaction.commit()

        content = ptah.cms.load(c_uri)
        self.assertEqual(content.__parent__.__uri__, co_uri)

    def test_loadapi_load_permission(self):
        import ptah

        allow = False
        def check_permission(permission, content, r=None, t=True):
            if not allow:
                return False

            return True

        # monkey patch
        orig_check_permission = ptah.check_permission
        ptah.check_permission = check_permission

        c = Content(title='Content')
        uri = c.__uri__
        ptah.cms.Session.add(c)
        transaction.commit()

        self.assertRaises(HTTPForbidden, ptah.cms.load, uri, 'View')

        allow = True
        c = ptah.cms.load(uri, 'View')
        self.assertEqual(c.__uri__, uri)

        # remove monkey patch
        ptah.check_permission = orig_check_permission



from zope import interface
from pyramid.security import authenticated_userid
from pyramid.threadlocal import get_current_request

from ptah import config
from ptah.uri import resolve, resolver
from ptah.util import tldata
from ptah.interfaces import IAuthInfo, IAuthentication


class _Superuser(object):

    def __init__(self):
        self.uri = 'ptah-auth:superuser'
        self.login = ''
        self.name = 'Manager'

    def __repr__(self):
        return '<ptah Superuser>'


SUPERUSER = _Superuser()
SUPERUSER_URI = 'ptah-auth:superuser'


@resolver('ptah-auth')
def superuser_resolver(uri):
    """System super user"""
    if uri == SUPERUSER_URI:
        return SUPERUSER


AUTH_CHECKER_ID = 'ptah.auth:checker'
AUTH_PROVIDER_ID = 'ptah.auth:provider'
AUTH_SEARCHER_ID = 'ptah.auth:searcher'


def auth_checker(checker):
    info = config.DirectiveInfo()
    info.attach(
        config.Action(
            lambda config: config.get_cfg_storage(AUTH_CHECKER_ID)\
                .update({id(checker): checker}),
            discriminator=(AUTH_CHECKER_ID, checker))
        )
    return checker


def register_auth_provider(name, provider):
    info = config.DirectiveInfo()

    info.attach(
        config.Action(
            lambda config, n, p: config.get_cfg_storage(AUTH_PROVIDER_ID)\
                .update({n: p}),
            (name, provider),
            discriminator=(AUTH_PROVIDER_ID, name))
        )


class AuthInfo(object):
    interface.implements(IAuthInfo)

    def __init__(self, status=False, principal=None, uri=None, message=u''):
        self.status = status
        self.message = message
        self.principal = principal
        self.uri = uri
        self.arguments = {}


_not_set = object()

USER_KEY = '__ptah_auth_userid__'


class Authentication(object):
    interface.implements(IAuthentication)

    def authenticate(self, credentials):
        info = AuthInfo()

        providers = config.get_cfg_storage(AUTH_PROVIDER_ID)
        for pname, provider in providers.items():
            principal = provider.authenticate(credentials)
            if principal is not None:
                info.uri = principal.uri
                info.principal = principal

                for checker in \
                        config.get_cfg_storage(AUTH_CHECKER_ID).values():
                    if not checker(info):
                        return info

                info.status = True
                return info

        return info

    def authenticate_principal(self, principal):
        info = AuthInfo()
        info.uri = principal.uri
        info.principal = principal

        for checker in \
                config.get_cfg_storage(AUTH_CHECKER_ID).values():
            if not checker(info):
                return info

        info.status = True
        return info

    def set_userid(self, uid):
        tldata.set(USER_KEY, uid)

    def get_userid(self):
        uid = tldata.get(USER_KEY, _not_set)
        if uid is _not_set:
            try:
                self.set_userid(authenticated_userid(get_current_request()))
            except:  # pragma: no cover
                self.set_userid(None)
            return tldata.get(USER_KEY)
        return uid

    def get_current_principal(self):
        return resolve(self.get_userid())

    def get_principal_bylogin(self, login):
        providers = config.get_cfg_storage(AUTH_PROVIDER_ID)

        for pname, provider in providers.items():
            principal = provider.get_principal_bylogin(login)
            if principal is not None:
                return principal

authService = Authentication()


def search_principals(term):
    searchers = config.get_cfg_storage(AUTH_SEARCHER_ID)
    for name, searcher in searchers.items():
        for principal in searcher(term):
            yield principal


def register_principal_searcher(name, searcher):
    info = config.DirectiveInfo()
    info.attach(
        config.Action(
            lambda config, name, searcher:
               config.get_cfg_storage(AUTH_SEARCHER_ID).update({name: searcher}),
            (name, searcher),
            discriminator=(AUTH_SEARCHER_ID, name))
        )


def principal_searcher(name):
    info = config.DirectiveInfo()

    def wrapper(searcher):
        info.attach(
            config.Action(
                lambda config, name, searcher:
                    config.get_cfg_storage(AUTH_SEARCHER_ID)\
                        .update({name: searcher}),
                (name, searcher),
                discriminator=(AUTH_SEARCHER_ID, name))
            )

        return searcher

    return wrapper



import decimal, datetime

from ptah import form


vocabulary = form.SimpleVocabulary.from_items(
    (1, 'one', 'One', 'One description'),
    (2, 'two', 'Two', 'Two description'),
    (3, 'three', 'Three', 'Three description'))


@form.fieldpreview(form.MultiChoiceField)
def multiChoicePreview(request):
    field = form.MultiChoiceField(
        'MultiChoiceField',
        title = 'Multi choice field',
        description = 'Multi choice field preview description',
        default = [1,3],
        vocabulary = vocabulary)

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.ChoiceField)
def choicePreview(request):
    field = form.ChoiceField(
        'ChoiceField',
        title = 'Choice field',
        description = 'Choice field preview description',
        missing = 1,
        vocabulary = vocabulary)

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.BoolField)
def boolPreview(request):
    field = form.BoolField(
        'BoolField',
        default = True,
        title = 'Boolean field',
        description = 'Boolean field preview description')

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.RadioField)
def radioPreview(request):
    field = form.RadioField(
        'RadioField',
        title = 'Radio field',
        description = 'Radio field preview description',
        default = 1,
        vocabulary = vocabulary)

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.TextAreaField)
def textareaPreview(request):
    field = form.TextAreaField(
        'TextAreaField',
        title = 'TextArea field',
        description = 'TextArea field preview description',
        default = 'Test text in text area field.')

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.LinesField)
def linesPreview(request):
    field = form.LinesField(
        'LinesField',
        title = 'Lines field',
        description = 'Lines field preview description',
        default = ['One', 'Two', 'Three'])

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.TextField)
def textPreview(request):
    field = form.TextField(
        'TextField',
        title = 'Text field',
        description = 'Text field preview description',
        default = 'Test text in text field.')

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.IntegerField)
def intPreview(request):
    field = form.IntegerField(
        'IntegerField',
        title = 'Integer field',
        description = 'Integer field preview description',
        default = 456782)

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.FloatField)
def floatPreview(request):
    field = form.FloatField(
        'FloatField',
        title = 'Float field',
        description = 'Float field preview description',
        default = 456782.236)

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.DecimalField)
def decimalPreview(request):
    field = form.DecimalField(
        'DecimalField',
        title = 'Decimal field',
        description = 'Decimal field preview description',
        default = decimal.Decimal('10.54'))

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.PasswordField)
def passwordPreview(request):
    field = form.PasswordField(
        'PasswordField',
        title = 'Password field',
        description = 'Password field preview description')

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.JSDateField)
def jsdatePreview(request):
    field = form.JSDateField(
        'JSDateField',
        title = 'jQuery Date field',
        description = 'jQuery Date field preview description',
        default = datetime.date.today())

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.JSDateTimeField)
def jsdatetimePreview(request):
    field = form.JSDateTimeField(
        'JSDateTimeField',
        title = 'jQuery DateTime field',
        description = 'jQuery DateTime field preview description')

    widget = field.bind('preview.', datetime.datetime.now(), {})
    widget.update(request)
    return widget.snippet('form-widget', widget)


@form.fieldpreview(form.TinymceField)
def tinemcePreview(request):
    field = form.TinymceField(
        'TinymceField',
        title = 'TinyMCE field',
        description = 'TinyMCE field preview description',
        default = 'Test text in tinymce field.',
        width = '200px')

    widget = field.bind('preview.', form.null, {})
    widget.update(request)
    return widget.snippet('form-widget', widget)



from ptah import config


class PrincipalEvent(object):
    """ base class for all principal related events """

    principal = None  # IPrincipal object

    def __init__(self, principal):  # pragma: no cover
        self.principal = principal


class LoggedInEvent(PrincipalEvent):
    """ User logged in to system."""
    config.event('Logged in event')


class LoginFailedEvent(PrincipalEvent):
    """ User login failed."""
    config.event('Login failed event')

    message = ''

    def __init__(self, principal, message=u''):  # pragma: no cover
        self.principal = principal
        self.message = message


class LoggedOutEvent(PrincipalEvent):
    """ User logged out."""
    config.event('Logged out event')


class ResetPasswordInitiatedEvent(PrincipalEvent):
    """ User has initiated password changeing."""
    config.event('Reset password initiated event')


class PrincipalPasswordChangedEvent(PrincipalEvent):
    """ User password has been changed. """
    config.event('User password has been changed')


class PrincipalValidatedEvent(PrincipalEvent):
    """ Principal account has been validated."""
    config.event('Account validation event')


class PrincipalAddedEvent(PrincipalEvent):
    """ Principal added event """


class PrincipalRegisteredEvent(PrincipalEvent):
    """ Principal registered event """



import sys, os
from setuptools import setup, find_packages

version='0.1'

install_requires = ['setuptools',
                    'sphinx',
                    'colander >= 0.9.4',
                    'pytz',
                    'iso8601',
                    'simplejson',
                    'chameleon == 2.4.5',
                    'pyramid',
                    'pyramid_tm',
                    'zope.interface >= 3.8.0',
                    'WebOb >= 1.2b2',
                    'SQLAlchemy',
                    'SQLAHelper',
                    'Pygments',
                    ]
tests_require = ['nose']


def read(f):
    return open(os.path.join(os.path.dirname(__file__), f)).read().strip()


setup(name='ptah',
      version=version,
      description=('Ptah is a fast, fun, open source high-level '
                   'Python web development environment.'),
      long_description='\n\n'.join((read('README.rst'), read('CHANGES.txt'))),
      classifiers=[
          'Framework :: Pylons',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'Programming Language :: Python',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: Internet :: WWW/HTTP :: WSGI'],
      author='Ptah Project',
      author_email='ptahproject@googlegroups.com',
      url='https://github.com/ptahproject/ptah/',
      license='BSD-derived',
      packages=find_packages(),
      install_requires = install_requires,
      tests_require = tests_require,
      test_suite = 'nose.collector',
      include_package_data = True,
      zip_safe = False,
      entry_points = {
        'ptah': ['package = ptah'],
        'console_scripts': [
            'settings = ptah.config.commands:settingsCommand',
            ],
        'paste.app_factory': [
            'app = ptah:make_wsgi_app'],
        'paste.paster_create_template': [
            'ptah101 = ptah.scaffolds:Ptah101ProjectTemplate',
            'ptah102 = ptah.scaffolds:Ptah102ProjectTemplate',
            'ptah201 = ptah.scaffolds:Ptah201ProjectTemplate',
            'ptah301 = ptah.scaffolds:Ptah301ProjectTemplate',
            ],
        'paste.global_paster_command': [
            'static = ptah.view.commands:StaticCommand',
            ],
        },
      )



""" ui actions """
import ptah
from ptah import config
from zope import interface


class IAction(interface.Interface):
    """ marker interface for actions """


class Action(object):
    interface.implements(IAction)

    id = ''
    title = ''
    description = ''
    action = ''
    action_factory = None
    condition = None
    permission = None
    sort_weight = 1.0,
    data = None

    def __init__(self, id='', **kw):
        self.id = id
        self.__dict__.update(kw)

    def url(self, context, request, url=''):
        if self.action_factory is not None:
            return self.action_factory(context, request)

        if self.action.startswith('/'):
            return '%s%s'%(request.application_url, self.action)
        else:
            return '%s%s'%(url, self.action)

    def check(self, context, request):
        if self.permission:
            if not ptah.check_permission(
                self.permission, context, request):
                return False

        if self.condition is not None:
            return self.condition(context, request)

        return True


def uiaction(context, id, title,
             description = '',
             action='', condition=None, permission=None,
             sort_weight = 1.0, **kw):

    kwargs = {'id': id,
              'title': title,
              'description': description,
              'condition': condition,
              'permission': permission,
              'sort_weight': sort_weight,
              'data': kw}

    if callable(action):
        kwargs['action_factory'] = action
    else:
        kwargs['action'] = action

    ac = Action(**kwargs)
    info = config.DirectiveInfo()

    info.attach(
        config.Action(
            lambda cfg, id, context, ac: \
                cfg.registry.registerAdapter(ac, (context,), IAction, id),
            (id, context, ac,),
            discriminator = ('ptah-cms:ui-action', id, context))
        )


def list_uiactions(content, request):
    url = request.resource_url(content)

    actions = []
    for name, action in request.registry.adapters.lookupAll(
        (interface.providedBy(content),), IAction):
        if action.check(content, request):
            actions.append(
                (action.sort_weight,
                 {'id': action.id,
                  'url': action.url(content, request, url),
                  'title': action.title,
                  'description': action.description,
                  'data': action.data}))

    actions.sort()
    return [ac for _w, ac in actions]



""" directives """
import imp
import inspect
import logging
import sys

from zope import interface
from pkgutil import walk_packages
from collections import OrderedDict

import api

log = logging.getLogger('ptah.config')


class EventDescriptor(object):

    #: Event name
    name = ''

    #: Event title
    title = ''

    #: Event category
    category = ''

    #: Event class or interface
    instance = None

    def __init__(self, inst, title, category):
        self.instance = inst
        self.title = title
        self.category = category
        self.description = inst.__doc__
        self.name = '%s.%s' % (inst.__module__, inst.__name__)


EVENT_ID = 'ptah.config:event'


def event(title='', category=''):
    """ Register event object, it is used for introspection only. """
    info = DirectiveInfo(allowed_scope=('class',))

    def descriminator(action):
        return (EVENT_ID, action.info.context)

    info.attach(
        ClassAction(
            _event, (title, category),
            discriminator=descriminator)
        )


def _event(cfg, klass, title, category):
    storage = cfg.get_cfg_storage(EVENT_ID)
    ev = EventDescriptor(klass, title, category)
    storage[klass] = ev
    storage[ev.name] = ev


def adapter(*required, **kw):
    """ Register adapter """
    info = DirectiveInfo()

    required = tuple(required)
    name = kw.get('name', '')

    def descriminator(action):
        return ('ptah.config:adapter',
                required, _getProvides(action.info.context), name)

    if info.scope in ('module', 'function call'):  # function decorator
        def wrapper(func):
            info.attach(
                Action(
                    _register,
                    ('registerAdapter', func, required), {'name': name},
                    discriminator=('ptah.config:adapter',
                                   required, _getProvides(func), name))
                )
            return func
        return wrapper
    else:
        info.attach(
            ClassAction(
                _adapts, (required, name),
                discriminator=descriminator)
            )


def subscriber(*required):
    """ Register event subscriber. """
    info = DirectiveInfo(allowed_scope=('module', 'function call'))

    def wrapper(func):
        info.attach(
            Action(
                _register, ('registerHandler', func, required),
                discriminator=('ptah.config:subscriber',
                                func, tuple(required)))
            )
        return func
    return wrapper


def _register(cfg, methodName, *args, **kw):
    method = getattr(cfg.registry, methodName)
    method(*args, **kw)


def _adapts(cfg, factory, required, name):
    cfg.registry.registerAdapter(factory, required, name=name)


def _getProvides(factory):
    p = list(interface.implementedBy(factory))
    if len(p) == 1:
        return p[0]
    else:
        raise TypeError(
            "The adapter factory doesn't implement a single interface "
            "and no provided interface was specified.")


ATTACH_ATTR = '__ptah_actions__'


class Action(object):

    hash = None

    def __init__(self, callable, args=(), kw={},
                 discriminator=None, order=0, info=None):
        self.callable = callable
        self.args = args
        self.kw = kw
        self.order = order
        self.info = info
        self._discriminator = discriminator

    def __hash__(self):
        return hash(self.hash)

    @property
    def discriminator(self):
        if callable(self._discriminator):
            return self._discriminator(self)
        return self._discriminator

    def __call__(self, cfg):
        if self.callable:
            try:
                self.callable(cfg, *self.args, **self.kw)
            except:  # pragma: no cover
                log.exception(self.discriminator)
                raise


class ClassAction(Action):

    def __call__(self, cfg):
        try:
            self.callable(cfg, self.info.context, *self.args, **self.kw)
        except:  # pragma: no cover
            log.exception(self.discriminator)
            raise


class DirectiveInfo(object):

    def __init__(self, depth=1, moduleLevel=False, allowed_scope=None):
        scope, module, f_locals, f_globals, codeinfo = \
            getFrameInfo(sys._getframe(depth + 1))

        if allowed_scope and scope not in allowed_scope:
            raise TypeError("This directive is not allowed "
                            "to run in this scope: %s" % scope)

        if scope == 'module':
            self.name = f_locals['__name__']
        else:
            self.name = codeinfo[2]

        self.locals = f_locals
        self.scope = scope
        self.module = module
        self.codeinfo = codeinfo

        api.mods.add(self.module.__name__)

        if depth > 1:
            _, mod, _, _, ci = getFrameInfo(sys._getframe(2))
            self.hash = (module.__name__, codeinfo[1], mod.__name__, ci[1])
        else:
            self.hash = (module.__name__, codeinfo[1])

    @property
    def context(self):
        if self.scope == 'module':
            return self.module
        else:
            return getattr(self.module, self.name, None)

    def attach(self, action):
        action.info = self
        if action.hash is None:
            action.hash = self.hash

        data = getattr(self.module, ATTACH_ATTR, None)
        if data is None:
            data = OrderedDict()
            setattr(self.module, ATTACH_ATTR, data)

        if action.hash in data:
            raise TypeError(
                "Directive registered twice: %s" % (action.discriminator,))
        data[action.hash] = action


def getFrameInfo(frame):
    """code from venusian package
    Return (kind,module,locals,globals) for a frame

    'kind' is one of "exec", "module", "class", "function call", or "unknown".
    """

    f_locals = frame.f_locals
    f_globals = frame.f_globals

    sameNamespace = f_locals is f_globals
    hasModule = '__module__' in f_locals
    hasName = '__name__' in f_globals

    sameName = hasModule and hasName
    sameName = sameName and f_globals['__name__'] == f_locals['__module__']

    module = hasName and sys.modules.get(f_globals['__name__']) or None

    namespaceIsModule = module and module.__dict__ is f_globals

    frameinfo = inspect.getframeinfo(frame)
    try:
        sourceline = frameinfo[3][0].strip()
    except:  # pragma: no cover
        sourceline = frameinfo[3]

    codeinfo = frameinfo[0], frameinfo[1], frameinfo[2], sourceline

    if not namespaceIsModule:  # pragma: no cover
        # some kind of funky exec
        kind = "exec"
    elif sameNamespace and not hasModule:
        kind = "module"
    elif sameName and not sameNamespace:
        kind = "class"
    elif not sameNamespace:
        kind = "function call"
    else:  # pragma: no cover
        # How can you have f_locals is f_globals, and have '__module__' set?
        # This is probably module-level code, but with a '__module__' variable.
        kind = "unknown"
    return kind, module, f_locals, f_globals, codeinfo


def scan(package, seen, exclude_filter=None):
    if isinstance(package, basestring):
        __import__(package)
        package = sys.modules[package]

    actions = []

    pkgname = package.__name__
    if pkgname in seen:
        return actions

    seen.add(pkgname)

    if hasattr(package, ATTACH_ATTR):
        actions.extend(getattr(package, ATTACH_ATTR).values())

    if hasattr(package, '__path__'):  # package, not module
        results = walk_packages(package.__path__, package.__name__ + '.')

        for importer, modname, ispkg in results:
            if modname in seen:  # pragma: no cover
                continue

            seen.add(modname)

            if exclude_filter is not None and modname != pkgname:
                if not exclude_filter(modname):
                    continue

            loader = importer.find_module(modname)
            if loader is not None:
                module_type = loader.etc[2]
                if module_type in (imp.PY_SOURCE, imp.PKG_DIRECTORY):
                    __import__(modname)
                    module = sys.modules[modname]
                    if hasattr(module, ATTACH_ATTR):
                        actions.extend(getattr(module, ATTACH_ATTR).values())

    return actions


def resolveConflicts(actions):
    # code from zope.configuration package

    # organize actions by discriminators
    unique = {}
    output = []
    for i in range(len(actions)):
        action = actions[i]
        discriminator = action.discriminator

        order = action.order or i
        a = unique.setdefault(discriminator, [])
        a.append((action.info.codeinfo[0], order, action))

    # Check for conflicts
    conflicts = {}
    for discriminator, dups in unique.items():
        # We need to sort the actions by the paths so that the shortest
        # path with a given prefix comes first:
        dups.sort()
        basepath, order, action = dups[0]

        output.append((order, action))
        if len(dups) > 1:
            conflicts[discriminator] = [
                action.info for _i1, _i2, action in dups]

    if conflicts:
        raise ConflictError(conflicts)

    # Now put the output back in the original order, and return it:
    r = []
    output.sort()
    for order, action in output:
        r.append(action)

    return r


class ConflictError(TypeError):

    def __init__(self, conflicts):
        self._conflicts = conflicts

    def __str__(self):
        r = ["Conflicting configuration actions\n"]
        items = self._conflicts.items()
        items.sort()
        for discriminator, infos in items:
            r.append("  For: %s\n" % (discriminator, ))
            for info in infos:
                filename, line, function, source = info.codeinfo
                s = ' File "%s", line %d, in %s\n' \
                    '      %s\n' % (filename, line, function, source)

                for line in unicode(s).rstrip().split(u'\n'):
                    r.append(u"    " + line)
                r.append(u'')

        return "\n".join(r)



from paste.httpserver import serve
import sqlalchemy as sqla
import ptah
from ptah import cms


class Hyperlink(cms.Content):
    __tablename__ = 'ptah_cms_hyperlink'
    __type__ = cms.Type('hyperlink', permission=cms.AddContent)
    href = sqla.Column(sqla.Unicode)


if __name__ == '__main__':
    app = ptah.make_wsgi_app({'settings':r'./ptah.ini'})
    # we are initialized after make_wsgi_app
    if not cms.Session.query(Hyperlink).first():
        link = Hyperlink(title='ptah project',
                         href='http://ptahproject.org')
        cms.Session.add(link)
        import transaction; transaction.commit()

    for link in cms.Session.query(Hyperlink).all():
        print 'curl http://localhost:8080/__rest__/cms/content:/%s' % link.__uri__

    serve(app, host='0.0.0.0')



import unittest


class TestViewLayersManager(unittest.TestCase):

    def test_register(self):
        from ptah.view.customize import _ViewLayersManager

        manager = _ViewLayersManager()

        manager.register('test', (1,))
        manager.register('test2', (1,))
        manager.register('', (1,))

        self.assertEqual(manager.layers[(1,)], ['', 'test', 'test2'])

    def test_enabled(self):
        from ptah.view.customize import _ViewLayersManager

        manager = _ViewLayersManager()

        manager.register('test', (1,))
        manager.register('test2', (1,))
        manager.register('', (1,))

        self.assertFalse(manager.enabled('test', (1,)))
        self.assertTrue(manager.enabled('test2', (1,)))
        self.assertFalse(manager.enabled('test', (2,)))



import transaction
from zope import interface
import ptah, ptah.cms
from ptah import config
from pyramid.testing import DummyRequest

from base import Base


class RestBase(Base):

    _allow = True

    def _check_perm(self, perm, content, request=None, throw=False):
        return self._allow

    def _setup_ptah(self):
        pass

    def _make_app(self):
        global ApplicationRoot
        class ApplicationRoot(ptah.cms.ApplicationRoot):
            __type__ = ptah.cms.Type('app')

        ApplicationRoot.__type__.cls = ApplicationRoot

        return ApplicationRoot

    def setUp(self):
        super(RestBase, self).setUp()

        self.orig_check_permission = ptah.check_permission
        ptah.check_permission = self._check_perm
        ptah.cms.ApplicationFactory._sql_get_root.reset()

    def tearDown(self):
        ptah.check_permission = self.orig_check_permission

        config.cleanup_system(self.__class__.__module__)
        super(RestBase, self).tearDown()


class TestRestApi(RestBase):

    def test_rest_srv(self):
        import ptah.rest
        self._init_ptah()

        services = config.get_cfg_storage(ptah.rest.REST_ID)

        self.assertIn('cms', services)

        srv = services['cms']
        self.assertEqual(srv.title, 'Ptah CMS API')
        self.assertEqual(srv.actions.keys(),
                         ['content', 'applications', 'apidoc', 'types'])

    def test_rest_applications(self):
        from ptah.cms.rest import cmsApplications

        ApplicationRoot = self._make_app()

        self._init_ptah()

        request = self._makeRequest()

        info = cmsApplications(request)
        self.assertEqual(info, [])

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/test', 'root', 'Root App', config=self.p_config)

        info = cmsApplications(request)
        self.assertEqual(len(info), 1)
        self.assertEqual(info[0]['__name__'], 'root')
        self.assertEqual(info[0]['__mount__'], 'test')
        self.assertEqual(info[0]['__link__'],
                         'http://localhost:8080/content:%s/%s/'%(
                info[0]['__mount__'], info[0]['__uri__']))

        ptah.cms.ApplicationFactory(
            ApplicationRoot, '/test2', 'root2', 'Root App',config=self.p_config)
        self.assertEqual(len(cmsApplications(request)), 2)

        self._allow = False
        self.assertEqual(len(cmsApplications(request)), 0)

    def test_rest_applications_default(self):
        from ptah.cms.rest import cmsApplications

        ApplicationRoot = self._make_app()

        self._init_ptah()

        request = self._makeRequest()

        info = cmsApplications(request)
        self.assertEqual(info, [])

        ptah.cms.ApplicationFactory(
            ApplicationRoot, '/', 'root', 'Root App', config=self.p_config)

        info = cmsApplications(request)
        self.assertEqual(len(info), 1)
        self.assertEqual(info[0]['__name__'], 'root')
        self.assertEqual(info[0]['__mount__'], '')
        self.assertEqual(
            info[0]['__link__'],
            'http://localhost:8080/content/%s/'%(info[0]['__uri__'],))

    def test_rest_types(self):
        from ptah.cms.rest import cmsTypes

        self._make_app()
        self._init_ptah()

        request = self._makeRequest()
        info = cmsTypes(request)

        self.assertEqual(info[0]['name'], 'app')
        self.assertEqual(info[0]['__uri__'], 'cms-type:app')
        self.assertEqual(len(info[0]['fieldset']), 2)
        self.assertEqual(info[0]['fieldset'][0]['name'], 'title')
        self.assertEqual(info[0]['fieldset'][1]['name'], 'description')

    def test_rest_content(self):
        from ptah.cms.rest import cmsContent
        ApplicationRoot = self._make_app()
        self._init_ptah()

        request = DummyRequest(
            subpath=('content:root',), environ=self._makeEnviron())
        self.assertRaises(ptah.cms.NotFound, cmsContent, request, 'root')

        request = DummyRequest(
            subpath=('content:test',), environ=self._makeEnviron())
        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/test', 'root', 'Root App', config=self.p_config)
        root = factory(request)
        root.__uri__ = 'cms-app:test'
        transaction.commit()

        self._allow = False
        self.assertRaises(ptah.cms.Forbidden, cmsContent, request, 'test')

        self._allow = True

        root = factory(request)

        info = cmsContent(request, 'test')
        self.assertEqual(info['__uri__'], root.__uri__)

        self.assertRaises(ptah.cms.NotFound,
                          cmsContent, request, 'test', action='unknown')

        info = cmsContent(request, 'test', root.__uri__)
        self.assertEqual(info['__uri__'], root.__uri__)

    def test_rest_content_default(self):
        from ptah.cms.rest import cmsContent
        ApplicationRoot = self._make_app()
        self._init_ptah()

        request = DummyRequest(subpath=('content',),environ=self._makeEnviron())

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/', 'root', 'Root App', config=self.p_config)
        root = factory(request)
        root.__uri__ = 'cms-app:test'
        transaction.commit()

        self._allow = False
        self.assertRaises(ptah.cms.Forbidden, cmsContent, request)

        self._allow = True

        root = factory(request)

        info = cmsContent(request)
        self.assertEqual(info['__uri__'], root.__uri__)

        self.assertRaises(ptah.cms.NotFound,
                          cmsContent, request, action='unknown')

        info = cmsContent(request, root.__uri__)
        self.assertEqual(info['__uri__'], root.__uri__)


class Content(ptah.cms.Content):

    __type__ = ptah.cms.Type('content', 'Test Content')
    __uri_factory__ = ptah.UriFactory('cms-content')


class Container(ptah.cms.Container):

    __type__ = ptah.cms.Type('container', 'Test Container')
    __uri_factory__ = ptah.UriFactory('cms-container')


class TestCMSRestAction(RestBase):

    def test_rest_cms_action(self):
        from ptah.cms.rest import IRestAction, IRestActionClassifier

        @ptah.cms.restaction('my-update', Content, ptah.cms.View)
        def update(content, request, *args):
            """ doc string """

        self._init_ptah()

        adapters = self.p_config.registry.adapters

        action = adapters.lookup(
            (IRestActionClassifier, interface.implementedBy(Content)),
            IRestAction, name='my-update')

        self.assertEqual(action.callable, update)
        self.assertEqual(action.name, 'my-update')
        self.assertEqual(action.description, update.__doc__)
        self.assertEqual(action.permission, ptah.cms.View)

    def test_rest_cms_action_conflicts(self):
        @ptah.cms.restaction('my-update', Content, ptah.cms.View)
        def update1(content, request, *args):
            """ doc string """

        @ptah.cms.restaction('my-update', Content, ptah.cms.View)
        def update2(content, request, *args):
            """ doc string """

        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_rest_cms_node_info(self):
        from ptah.cms import rest
        self._init_ptah()

        content = Content()
        request = self._makeRequest()
        info = rest.nodeInfo(content, request)

        self.assertEqual(info['__uri__'], content.__uri__)
        self.assertEqual(info['__link__'],
                         '%s%s/'%(request.application_url, content.__uri__))

    def test_rest_cms_apidoc(self):
        from ptah.cms import rest
        self._init_ptah()

        content = Content()
        request = self._makeRequest()
        request.registry = self.p_config.registry
        info = rest.apidocAction(content, request)
        self.assertEqual(len(info), 5)
        self.assertEqual(info[0]['name'], 'info')

        self._allow = False
        info = rest.apidocAction(content, request)
        self.assertEqual(len(info), 0)

    def test_rest_cms_container_info(self):
        from ptah.cms import rest
        self._init_ptah()

        container = Container()
        container['content'] = Content()

        request = self._makeRequest()
        info = rest.containerNodeInfo(container, request)

        self.assertEqual(info['__uri__'], container.__uri__)
        self.assertEqual(info['__link__'],
                         '%s%s/'%(request.application_url, container.__uri__))
        self.assertEqual(len(info['__contents__']), 1)
        self.assertEqual(info['__contents__'][0]['__uri__'],
                         container['content'].__uri__)

    def test_rest_cms_delete(self):
        from ptah.cms import rest
        self._init_ptah()

        container = Container()
        container['content'] = Content()

        request = self._makeRequest()
        rest.deleteAction(container['content'], request)
        self.assertEqual(container.keys(), [])

    def test_rest_cms_update(self):
        from ptah.cms import rest
        self._init_ptah()

        content = Content(title='Test')

        request = DummyRequest(params = {'title':''})
        info = rest.updateAction(content, request)
        self.assertEqual(info['errors']['title'], 'Required')

        request = DummyRequest(params = {'title':'New title'})

        info = rest.updateAction(content, request)
        self.assertEqual(
            info['__link__'],
            '%s%s/'%(request.application_url, content.__uri__))
        self.assertEqual(content.title, 'New title')

    def test_rest_cms_create(self):
        from ptah.cms import rest
        self._init_ptah()

        all_types = ptah.cms.get_types()

        all_types[Content.__type__.__uri__] = Content.__type__

        container = Container()

        request = DummyRequest(
            params = {'tinfo': Content.__type__.__uri__, 'name': 'content'},
            post = {'title':''})
        info = rest.createContentAction(container, request)
        self.assertEqual(info['errors']['title'], 'Required')

        request = DummyRequest(
            params = {'tinfo': Content.__type__.__uri__, 'name': 'content'},
            post = {'title':'New title'})

        rest.createContentAction(container, request)
        self.assertEqual(container.keys(), ['content'])
        self.assertEqual(container['content'].title, 'New title')



""" Basic fields """
import datetime
import decimal
import iso8601

from ptah import view
from ptah.form import vocabulary
from ptah.form.field import field, Field
from ptah.form.interfaces import _, null, Invalid, ITerm


class InputField(Field, view.View):

    title = None
    lang = None
    disabled = None
    tabindex = None
    lang = None
    disabled = None
    readonly = None
    alt = None
    accesskey = None
    size = None
    maxlength = None

    def serialize(self, value):
        return value

    def deserialize(self, value):
        return value

    def update(self, request):
        super(InputField, self).update(request)

        if self.readonly:
            self.add_css_class('disabled')

    def add_css_class(self, css):
        self.klass = ('%s %s' % (self.klass or '', css)).strip()


class VocabularyField(InputField):

    vocabulary = None
    noValueToken = '--NOVALUE--'

    def __init__(self, name, **kw):
        super(VocabularyField, self).__init__(name, **kw)

        if self.vocabulary is None:
            raise ValueError("Vocabulary is not specified.")

    def is_checked(self, term):
        raise NotImplementedError()

    def update_items(self):
        self.items = []
        for count, term in enumerate(self.vocabulary):
            checked = self.is_checked(term)
            id = '%s-%i' % (self.id, count)
            label = term.token
            desc = None
            if ITerm.providedBy(term):
                label = term.title
                desc = term.description
            self.items.append(
                {'id': id, 'name': self.name, 'value': term.token,
                 'label': label, 'description': desc, 'checked': checked})


class BaseChoiceField(VocabularyField):
    """ choice field """

    tmpl_display = view.template(
        "ptah.form:templates/fields/basechoice-display.pt")

    def serialize(self, value):
        if value is null:
            return null

        try:
            return self.vocabulary.get_term(value).token
        except Exception:
            raise Invalid(
                self, _('"${val}" is not in vocabulary',
                        mapping={'val': value}))

    def deserialize(self, value):
        if not value:
            return null

        try:
            return self.vocabulary.get_term_bytoken(value).value
        except Exception:
            raise Invalid(
                self, _('"${val}" is not in vocabulary',
                        mapping={'val': value}))

    def extract(self, default=null):
        value = self.params.get(self.name, default)
        if value is default:
            return default

        if value == self.noValueToken:
            return default

        return value

    def is_checked(self, term):
        return term.token == self.form_value

    def update(self, request):
        super(BaseChoiceField, self).update(request)

        self.update_items()


class BaseMultiChoiceField(VocabularyField):
    """ multi choice field """

    tmpl_display = view.template(
        "ptah.form:templates/fields/basemultichoice-display.pt")

    def serialize(self, value):
        if value is null:
            return null
        try:
            res = []
            for val in value:
                res.append(self.vocabulary.get_term(val).token)
            return res
        except (LookupError, TypeError):
            raise Invalid(
                self, _('"${val}" is not in vocabulary',
                        mapping={'val': value}))

    def deserialize(self, value):
        if not value:
            return null
        try:
            res = []
            for val in value:
                res.append(self.vocabulary.get_term_bytoken(val).value)
            return res
        except Exception:
            raise Invalid(
                self, _('"${val}" is not in vocabulary',
                        mapping={'val': value}))

    def extract(self, default=null):
        if self.name not in self.params:
            return default

        value = []
        tokens = self.params.getall(self.name)
        for token in tokens:
            if token == self.noValueToken:
                continue

            value.append(token)

        return value

    def is_checked(self, term):
        return term.token in self.form_value

    def update(self, request):
        super(BaseMultiChoiceField, self).update(request)

        if self.form_value in (null, None):
            self.form_value = []

        self.update_items()


class TextField(InputField):
    __doc__ = _(u'HTML Text input widget')

    field('text')

    klass = u'text-widget'
    value = u''

    tmpl_input = view.template(
        "ptah.form:templates/fields/text-input.pt")
    tmpl_display = view.template(
        "ptah.form:templates/fields/text-display.pt")


class Number(object):

    num = None

    def serialize(self, value):
        if value is null:
            return null

        try:
            return str(self.num(value))
        except Exception:
            raise Invalid(self,
                          _('"${val}" is not a number',
                            mapping={'val': value}),
                          )

    def deserialize(self, value):
        if value != 0 and not value:
            return null

        try:
            return self.num(value)
        except Exception:
            raise Invalid(
                self, _('"${val}" is not a number', mapping={'val': value}))


class IntegerField(Number, TextField):
    __doc__ = _(u'Integer input widget')

    field('int')

    klass = u'int-widget'
    value = 0

    num = int


class FloatField(Number, TextField):
    __doc__ = _(u'Float input widget')

    field('float')

    klass = u'float-widget'

    num = float


class DecimalField(Number, TextField):
    __doc__ = _(u'Decimal input widget')

    field('decimal')

    klass = u'decimal-widget'

    def num(self, val):
        return decimal.Decimal(str(val))


class TextAreaField(TextField):
    __doc__ = _(u'HTML Text Area input widget')

    field('textarea')

    klass = u'textarea-widget'
    value = u''

    rows = 5
    cols = 40

    tmpl_input = view.template(
        "ptah.form:templates/fields/textarea-input.pt")


class FileField(TextField):
    __doc__ = _(u'HTML File input widget')

    field('file')
    klass = u'input-file'

    tmpl_input = view.template(
        "ptah.form:templates/fields/file-input.pt")

    def extract(self, default=null):
        value = self.params.get(self.name, default)

        if hasattr(value, 'file'):
            data = {}
            data['fp'] = value.file
            data['filename'] = value.filename
            data['mimetype'] = value.type
            data['size'] = value.length
            return data

        return default


class LinesField(TextAreaField):
    __doc__ = _('Text area based widget, each line is treated as '
                'sequence element.')

    field('lines')
    klass = u'textlines-widget'

    def serialize(self, value):
        if value is null or not value:
            return null

        try:
            return u'\n'.join(value)
        except Exception:
            raise Invalid(self,
                          _('"${val}" is not a list',
                            mapping={'val': value}),
                          )

    def deserialize(self, value):
        if not value:
            return null

        try:
            return [s.strip() for s in value.split()]
        except Exception:
            raise Invalid(self,
                          _('"${val}" is not a list',
                            mapping={'val': value}),
                          )


class PasswordField(TextField):
    __doc__ = _('HTML Password input widget.')

    field('password')

    klass = u'password-widget'

    tmpl_input = view.template(
        "ptah.form:templates/fields/password-input.pt")
    tmpl_display = view.template(
        "ptah.form:templates/fields/password-display.pt")


class MultiChoiceField(BaseMultiChoiceField):
    __doc__ = _('HTML Checkboxs input based widget.')

    field('multichoice')

    klass = u'multichoice-widget'
    tmpl_input = view.template(
        "ptah.form:templates/fields/multichoice-input.pt")


class DateField(TextField):
    __doc__ = _(u'Simple date input field.')

    tmpl_display = view.template(
        "ptah.form:templates/fields/date-display.pt")

    def serialize(self, value):
        if value is null:
            return null

        if isinstance(value, datetime.datetime):
            value = value.date()

        if not isinstance(value, datetime.date):
            raise Invalid(self,
                          _('"${val}" is not a date object',
                            mapping={'val': value}))

        return value.isoformat()

    def deserialize(self, value):
        if not value:
            return null
        try:
            result = iso8601.parse_date(value)
            result = result.date()
        except (iso8601.ParseError, TypeError):
            try:
                year, month, day = map(int, value.split('-', 2))
                result = datetime.date(year, month, day)
            except Exception, e:
                raise Invalid(
                    self, _('Invalid date', mapping={'val': value, 'err': e}))

        return result


class DateTimeField(TextField):

    default_tzinfo = iso8601.iso8601.Utc()

    tmpl_display = view.template(
        "ptah.form:templates/fields/datetime-display.pt")

    def serialize(self, value):
        if value is null or value is None:
            return null

        if type(value) is datetime.date:  # cannot use isinstance; dt subs date
            value = datetime.datetime.combine(value, datetime.time())

        if not isinstance(value, datetime.datetime):
            raise Invalid(
                self, _('"${val}" is not a datetime object',
                        mapping={'val': value}))

        if value.tzinfo is None:
            value = value.replace(tzinfo=self.default_tzinfo)

        return value.isoformat()

    def deserialize(self, value):
        if not value:
            return null

        try:
            result = iso8601.parse_date(
                value, default_timezone=self.default_tzinfo)
        except (iso8601.ParseError, TypeError), e:
            try:
                year, month, day = map(int, value.split('-', 2))
                result = datetime.datetime(year, month, day,
                                           tzinfo=self.default_tzinfo)
            except Exception, e:
                raise Invalid(self, _('Invalid date',
                                      mapping={'val': value, 'err': e}))
        return result


class RadioField(BaseChoiceField):
    __doc__ = _('HTML Radio input widget.')

    field('radio')

    klass = u'radio-widget'
    tmpl_input = view.template(
        "ptah.form:templates/fields/radio-input.pt")


class BoolField(BaseChoiceField):
    __doc__ = _('Boolean input widget.')

    field('bool')

    vocabulary = vocabulary.SimpleVocabulary.from_items(
        (True, 'true',  _('yes')),
        (False, 'false',  _('no')))

    tmpl_input = view.template(
        "ptah.form:templates/fields/bool-input.pt")


class ChoiceField(BaseChoiceField):
    __doc__ = _('HTML Select input widget.')

    field('choice')

    size = 1
    klass = u'select-widget'
    multiple = None
    promptMessage = _('select a value ...')

    tmpl_input = view.template(
        "ptah.form:templates/fields/select-input.pt")

    def update_items(self):
        super(ChoiceField, self).update_items()

        if not self.required:
            self.items.insert(0, {
                    'id': self.id + '-novalue',
                    'name': self.name,
                    'value': self.noValueToken,
                    'label': self.promptMessage,
                    'checked': self.form_value is null,
                    'description': u'',
                    })


class MultiSelectField(ChoiceField):
    __doc__ = _('HTML Multi Select input widget.')

    size = 5
    multiple = 'multiple'

    field('multiselect')



import os
from paste.script.templates import Template
from paste.util.template import paste_script_template_renderer


class PtahTemplate(Template):

    def post(self, command, output_dir, vars): # pragma: no cover
        print ("\n\nPtah has generated your application. Your application "
               "can be found at /. \n\nDorneles hopes you like it.\n")

        return super(PtahTemplate, self).post(command, output_dir, vars)


class Ptah101ProjectTemplate(PtahTemplate):
    _template_dir = 'ptah101'
    summary = 'demonstrates form, limited mgmt ui'
    template_renderer = staticmethod(paste_script_template_renderer)


class Ptah102ProjectTemplate(PtahTemplate):
    _template_dir = 'ptah102'
    summary = 'demonstrates form, models, limited mgmt ui'
    template_renderer = staticmethod(paste_script_template_renderer)


class Ptah201ProjectTemplate(PtahTemplate):
    _template_dir = 'ptah201'
    summary = 'demonstrates form, models, limited mgmt ui and auth'
    template_renderer = staticmethod(paste_script_template_renderer)


class Ptah301ProjectTemplate(PtahTemplate):
    _template_dir = 'ptah301'
    summary = 'demonstrates traversal, localroles, full mgmt ui'
    template_renderer = staticmethod(paste_script_template_renderer)



""" view tests """
from zope import interface
from pyramid.interfaces import IView, IRequest, IRouteRequest
from pyramid.interfaces import IViewClassifier
from pyramid.interfaces import IExceptionViewClassifier
from pyramid.interfaces import IAuthorizationPolicy
from pyramid.interfaces import IAuthenticationPolicy
from pyramid.httpexceptions import HTTPForbidden, HTTPFound

from ptah import config, view

from base import Base, Context


class BaseView(Base):

    def _setup_ptah(self):
        pass

    def _view(self, name, context, request):
        adapters = self.registry.adapters

        view_callable = adapters.lookup(
            (IViewClassifier,
             interface.providedBy(request),
             interface.providedBy(context)),
            IView, name=name, default=None)

        return view_callable(context, request)


class TestView(BaseView):

    def test_view_register_errs(self):
        self.assertRaises(
            ValueError, view.register_view, 'test.html', None)

        self.assertRaises(
            ValueError, view.register_view, 'test.html', {})

    def test_view_register_view(self):
        class MyView(view.View):
            def render(self):
                return '<html>view</html>'

        view.register_view('index.html', MyView)
        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.status, '200 OK')
        self.assertEqual(v.content_type, 'text/html')
        self.assertEqual(v.body, '<html>view</html>')

    def test_view_register_declarative(self):
        global MyView

        class MyView(view.View):
            view.pview('index.html')

            def render(self):
                return '<html>view</html>'

        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.status, '200 OK')
        self.assertEqual(v.content_type, 'text/html')
        self.assertEqual(v.body, '<html>view</html>')

    def test_view_register_view_err1(self):
        # default 'render' implementation
        class MyView(view.View):
            pass

        view.register_view('index.html', MyView, Context)
        self._init_ptah()

        context = Context()
        self.assertTrue(
            view.render_view('index.html', context,
                            self.request).content_length ==0)

    def test_view_register_view_layout(self):
        class MyLayout(view.Layout):
            def render(self, rendered):
                return '<html>%s</html>'%rendered

        class MyView(view.View):
            def render(self):
                return 'test'

        view.register_view('index.html', MyView, Context)
        view.register_layout('', Context, klass=MyLayout)
        self._init_ptah()

        context = Context()
        res = view.render_view('index.html', context, self.request)
        self.assertTrue('<html>test</html>' in res.body)

    def test_view_register_view_disable_layout1(self):
        class MyLayout(view.Layout):
            def render(self, rendered):
                return '<html>%s</html>'%rendered

        class MyView(view.View):
            def render(self):
                return 'test'

        view.register_view('index.html', MyView, Context, layout=None)
        view.register_layout('', Context, klass=MyLayout)
        self._init_ptah()

        context = Context()
        res = view.render_view('index.html', context, self.request)
        self.assertEqual(res.body, 'test')

        v = MyView(None, self.request)
        self.assertEqual(MyLayout(v, self.request).render(
                v.render()), '<html>test</html>')

    def test_view_custom_response(self):
        class MyView(view.View):
            def render(self):
                response = self.request.response
                response.status = '202'
                return 'test'

        view.register_view('index.html', MyView, Context)
        self._init_ptah()

        res = view.render_view('index.html', Context(), self.request)
        self.assertEqual(res.status, '202 Accepted')
        self.assertEqual(res.body, 'test')

    def test_view_custom_return_response(self):
        class MyView(view.View):
            def render(self):
                response = self.request.response
                response.status = '202'
                response.body = 'test response'
                return response

        view.register_view('index.html', MyView, Context)
        self._init_ptah()

        res = view.render_view('index.html', Context(), self.request)
        self.assertEqual(res.status, '202 Accepted')
        self.assertEqual(res.body, 'test response')

    def test_view_httpresp_from_update(self):
        class MyView(view.View):
            def update(self):
                raise HTTPForbidden()

        view.register_view('index.html', MyView, Context,
                          template = view.template('templates/test.pt'))
        self._init_ptah()

        resp = view.render_view('index.html', Context(), self.request)
        self.assertIsInstance(resp, HTTPForbidden)

    def test_view_httpresp_from_update_return(self):
        class MyView(view.View):
            def update(self):
                return HTTPForbidden()

        view.register_view('index.html', MyView, Context,
                          template = view.template('templates/test.pt'))
        self._init_ptah()

        resp = view.render_view('index.html', Context(), self.request)
        self.assertIsInstance(resp, HTTPForbidden)

    def test_view_httpresp_from_render(self):
        class MyView(view.View):
            def render(self):
                raise HTTPFound()

        view.register_view('index.html', MyView, Context)
        self._init_ptah()

        resp = view.render_view('index.html', Context(), self.request)
        self.assertIsInstance(resp, HTTPFound)

    def test_view_with_template(self):
        view.register_view(
            'index.html', view.View, Context,
            template=view.template('ptah.view.tests:templates/test.pt'))

        self._init_ptah()

        res = view.render_view('index.html', Context(), self.request)
        self.assertEqual(res.body.strip(), '<div>My snippet</div>')

    def test_view_with_decorator(self):
        def deco(func):
            def func(context, request):
                return 'decorator'
            return func

        global DecoView

        @deco
        class DecoView(view.View):
            view.pview('index.html', Context)

        self._init_ptah()

        res = view.render_view('index.html', Context(), self.request)
        self.assertEqual(res.body, 'decorator')

    def test_view_register_view_class_requestonly(self):
        class MyView(object):
            def __init__(self, request):
                self.request = request

            def render(self):
                return '<html>view: %s</html>'%(self.request is not None)

        view.register_view('index.html', MyView)
        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>view: True</html>')

    def test_view_register_view_function(self):
        def render(context, request):
            return '<html>context: %s</html>'%(context is not None)

        view.register_view('index.html', render)
        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>context: True</html>')

    def test_view_register_view_function_requestonly(self):
        def render(request):
            return '<html>request: %s</html>'%(request is not None)

        view.register_view('index.html', render)
        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>request: True</html>')

    def test_view_register_view_function_with_template(self):
        def render(context, request):
            return {}

        view.register_view('index.html', render,
                          template = view.template('templates/test.pt'))
        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body.strip(), '<div>My snippet</div>')

    def test_view_register_view_function_requestonly_template(self):
        def render(request):
            return {}

        view.register_view('index.html', render,
                          template = view.template('templates/test.pt'))
        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body.strip(), '<div>My snippet</div>')

    def test_view_register_callable_permission(self):
        def render(request):
            return '<html>Secured view</html>'

        allowed = False
        def check_permission(context, request):
            return allowed

        view.register_view('index.html', render,
                          permission = check_permission)

        self._init_ptah()

        context = Context()

        resp = self._view('index.html', context, self.request)

        self.assertIsInstance(resp, HTTPForbidden)

        allowed = True
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>Secured view</html>')

    def test_view_register_secured_view(self):
        from ptah.view.renderers import \
            default_checkpermission, set_checkpermission

        def render(request):
            return '<html>Secured view</html>'

        view.register_view('index.html', render,
                          permission = 'Protected')

        self._init_ptah()

        set_checkpermission(default_checkpermission)

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>Secured view</html>')

        class SimpleAuth(object):
            interface.implements(IAuthenticationPolicy)

            def effective_principals(self, request):
                return (1,2)

        class Authz(object):
            interface.implements(IAuthorizationPolicy)

            allowed = False

            def permits(self, context, princials, permission):
                return self.allowed

        self.registry.registerUtility(SimpleAuth(), IAuthenticationPolicy)
        self.registry.registerUtility(Authz(), IAuthorizationPolicy)

        set_checkpermission(default_checkpermission)

        context = Context()
        resp = self._view('index.html', context, self.request)
        self.assertIsInstance(resp, HTTPForbidden)

        Authz.allowed = True
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>Secured view</html>')

    def test_view_function(self):
        @view.pview('index.html')
        def render(request):
            return '<html>content</html>'

        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, '<html>content</html>')

    def test_view_custom_class(self):
        global View
        class View(object):
            view.pview('index.html')

            def __init__(self, request):
                self.request = request
                self.updated = False
            def update(self):
                self.updated = True
            def render(self):
                return str(self.updated)

        self._init_ptah()

        context = Context()
        v = self._view('index.html', context, self.request)
        self.assertEqual(v.body, 'True')

    def test_view_for_exception(self):
        @view.pview(context=HTTPForbidden, layer='test')
        def render(request):
            return '<html>Forbidden</html>'

        self._init_ptah()

        context = HTTPForbidden()

        adapters = self.registry.adapters

        view_callable = adapters.lookup(
            (IExceptionViewClassifier,
             interface.providedBy(self.request),
             interface.providedBy(context)),
            IView, name='', default=None)

        v = view_callable(context, self.request)
        self.assertEqual(v.body, '<html>Forbidden</html>')

    def test_view_for_route(self):
        view.register_route('test-route', '/test/')

        @view.pview(route='test-route')
        def render(request):
            return '<html>Route view</html>'

        self._init_ptah()

        request_iface = self.registry.getUtility(
            IRouteRequest, name='test-route')

        interface.directlyProvides(self.request, request_iface)

        v = self._view('', None, self.request)
        self.assertEqual(v.body, '<html>Route view</html>')


class TestRouteRegistration(BaseView):

    def test_view_route(self):
        view.register_route('test-route', '/test/')
        self._init_ptah()

        request_iface = self.registry.getUtility(
            IRouteRequest, name='test-route')

        self.assertIsNotNone(request_iface)

    def test_view_route_global_view(self):
        view.register_route('test-route', '/test/', use_global_views=True)
        self._init_ptah()

        request_iface = self.registry.getUtility(
            IRouteRequest, name='test-route')

        self.assertTrue(request_iface.isOrExtends(IRequest))

    def test_view_route_conflict(self):
        view.register_route('test-route', '/test/')
        view.register_route('test-route', '/test2/')
        self.assertRaises(config.ConflictError, self._init_ptah)


class TestViewView(BaseView):

    def test_view_render(self):

        class MyView(view.View):
            def template(self, **kw):
                return 'MyView rendered'

        self.assertEqual(MyView(None, None).render(), 'MyView rendered')



""" content traverser """
import ptah
from ptah import config
from sqlalchemy import sql
from zope import interface
from pyramid.interfaces import ITraverser
from pyramid.traversal import traversal_path, ResourceTreeTraverser

from node import Session
from content import Content
from interfaces import IApplicationRoot


class ContentTraverser(object):
    interface.implements(ITraverser)
    config.adapter(IApplicationRoot)

    _path_queries = {}

    def __init__(self, root):
        self.root = root

    def __call__(self, request, queries=_path_queries):
        environ = request.environ
        context = root = self.root

        if root.__default_root__ and 'bfg.routes.route' in environ:
            return ResourceTreeTraverser(root)(request)

        path = '/%s/'%'/'.join(traversal_path(environ.get('PATH_INFO','/')))

        vroot_tuple = ()

        l_path = len(root.__root_path__)

        # build paths for all parents in content hierarchy
        idx = 0
        paths = {}
        current = root.__path__
        for sec in path[l_path:].split('/'):
            if sec:
                current = '%s%s/'%(current, sec)
                paths[str(idx)] = current
                idx += 1

        if idx:
            if idx not in queries:
                bindparams = [sql.bindparam(str(p)) for p in range(idx)]

                queries[idx] = ptah.QueryFreezer(
                    lambda: Session.query(Content)\
                        .filter(Content.__path__.in_(bindparams))
                        .order_by(sql.desc(Content.__path__)))

            parents = queries[idx].all(**paths)
        else:
            parents = []

        if parents:
            # set __parent__
            parents[-1].__parent__ = root
            for idx in range(len(parents)-1):
                parents[idx].__parent__ = parents[idx+1]

            context = parents[0]
            node = context.__path__[len(root.__path__):]
            leaf = path[l_path+len(node):].split('/')
            leaf, subpath = leaf[0], leaf[1:]

            return {'context': context,
                    'view_name': leaf,
                    'subpath': subpath,
                    'traversed': traversal_path(node),
                    'virtual_root': root,
                    'virtual_root_path': vroot_tuple,
                    'root': root}
        else:
            vpath_tuple = ()

            leaf = path[l_path:].split('/')
            leaf, subpath = leaf[0], leaf[1:]

            return {'context': context,
                    'view_name': leaf,
                    'subpath': subpath,
                    'traversed': vpath_tuple,
                    'virtual_root': root,
                    'virtual_root_path': (),
                    'root': root}



""" resource library """
from urlparse import urlparse
from ptah import config
from ptah.view.resources import static_url

LIBRARY_ID = 'ptah.view:library'


def library(name,
            path='', resource='', type='',
            require='', prefix='', postfix='', extra=None):

    if not path:
        raise ValueError("path is required")

    if type not in ('js', 'css'):
        raise ValueError("Uknown type '%s'"%type)

    if isinstance(path, basestring):
        path = (path,)

    if require and isinstance(require, basestring):
        require = (require,)

    if not resource:
        for p in path:
            if not urlparse(p)[0]:
                raise ValueError("If resource is not defined "
                                 "path has to be absolute url")

    info = config.DirectiveInfo()

    info.attach(
        config.Action(
            library_impl,
            (name, path, resource, type,
             require, prefix, postfix, extra),
            discriminator = (LIBRARY_ID, name, tuple(path), resource))
        )


def library_impl(cfg, name, path, resource, type,
                 require, prefix, postfix, extra):
    data = cfg.get_cfg_storage(LIBRARY_ID)

    lib = data.get(name)
    if lib is None:
        lib = Library(name)
        data[name] = lib

    if extra is None:
        extra = {}

    if path:
        lib.add(path, resource, type, require, prefix, postfix, extra)


def include(request, name):
    libs = getattr(request, '__includes', None)
    if libs is None:
        libs = []

    libs.append(name)
    request.__includes = libs


def render_includes(request):
    _libraries = config.get_cfg_storage(LIBRARY_ID)

    seen = set()
    libraries = []

    libs = getattr(request, '__includes', None)
    if libs is None:
        return u''

    def _process(l_id):
        lib = _libraries.get(l_id)
        if lib is None:
            seen.add(l_id)
            return

        for require in lib.require:
            if require not in seen:
                _process(require)

        seen.add(l_id)
        libraries.append(lib)

    for id in libs:
        if id not in seen:
            _process(id)

    return u'\n'.join(lib.render(request) for lib in libraries)


class Entry(object):

    def __init__(self, path, resource='',
                 type='', prefix='', postfix='', extra={}):
        self.resource = resource
        self.type = type
        self.prefix = prefix
        self.postfix = postfix

        if type == 'css':
            if 'rel' not in extra:
                extra['rel'] = 'stylesheet'
            if 'type' not in extra:
                extra['type'] = 'text/css'

        self.extra = extra

        self.urls = urls = []
        self.paths = paths = []
        for p in path:
            if urlparse(p)[0]:
                urls.append(p)
            else:
                paths.append(p)

    def render(self, request):
        urls = list(self.urls)

        for path in self.paths:
            urls.append(static_url(request, self.resource, path))

        if self.type == 'css':
            s = '<link %shref="%s" />'
        else:
            s = '<script %ssrc="%s"> </script>'

        extra = ' '.join('%s="%s"'%(k,v) for k, v in self.extra.items())
        if extra:
            extra = '%s '%extra

        return '%s%s%s'%(
            self.prefix, '\n\n\t'.join(s%(extra, url) for url in urls),
            self.postfix)


class Library(object):

    def __init__(self, name):
        self.name = name
        self.require = []
        self.entries = []

    def add(self, path, resource, type, require, prefix, postfix, extra):
        self.entries.append(Entry(path,resource,type,prefix,postfix,extra))

        for req in require:
            if req not in self.require:
                self.require.append(req)

    def render(self, request):
        return '\n'.join(entry.render(request) for entry in self.entries)

    def __repr__(self):
        return '<ptah.view.library.Library "%s">'%self.name



import decimal
import unittest
from ptah import form
from webob.multidict import MultiDict

from base import Base


class DummyRequest(object):
    def __init__(self):
        self.params = {}
        self.cookies = {}


def invalid_exc(func, *arg, **kw):
    from ptah.form import Invalid
    try:
        func(*arg, **kw)
    except Invalid, e:
        return e
    else:
        raise AssertionError('Invalid not raised') # pragma: no cover


def strip(str):
    return ' '.join(s.strip() for s in str.split())


class TestInputField(Base):

    def _makeOne(self, name, **kw):
        from ptah.form.fields import InputField
        return InputField(name, **kw)

    def test_fields_text(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', u'content', {})
        field.update(request)

        self.assertEqual(field.klass, None)

        field.readonly = True
        field.update(request)

        self.assertEqual(field.klass, 'disabled')


class TestTextField(Base):

    def _makeOne(self, name, **kw):
        return form.TextField(name, **kw)

    def test_fields_text(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', u'content', {})
        field.update(request)

        self.assertEqual(field.serialize(u'value'), u'value')
        self.assertEqual(field.deserialize(u'value'), u'value')

        self.assertEqual(strip(field.render(request)),
                         '<input type="text" name="test" title="Test" value="content" id="test" class="text-widget" />')

        field.mode = form.FORM_DISPLAY
        self.assertEqual(strip(field.render(request)),
                         '<span class="uneditable-input" id="test" title="Test"> content </span>')


        field = self._makeOne('test')
        field = field.bind('', u'content', {'test': 'form'})
        field.update(request)

        self.assertEqual(strip(field.render(request)),
                         '<input type="text" name="test" title="Test" value="form" id="test" class="text-widget" />')

        field.mode = form.FORM_DISPLAY
        self.assertEqual(strip(field.render(request)),
                         '<span class="uneditable-input" id="test" title="Test"> form </span>')


class TestIntegerField(Base):

    def _makeOne(self, name, **kw):
        return form.IntegerField(name, **kw)

    def test_fields_int(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', 10, {})
        field.update(request)

        self.assertIs(field.serialize(form.null), form.null)
        self.assertEqual(field.serialize(10), '10')
        self.assertRaises(form.Invalid, field.serialize, u'value')

        self.assertIs(field.deserialize(''), form.null)
        self.assertEqual(field.deserialize('10'), 10)
        self.assertRaises(form.Invalid, field.deserialize, u'value')

        self.assertEqual(
            strip(field.render(request)),
            '<input type="text" name="test" title="Test" value="10" id="test" class="int-widget" />')

class TestFloatField(Base):

    def _makeOne(self, name, **kw):
        return form.FloatField(name, **kw)

    def test_fields_float(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', 10.34, {})
        field.update(request)

        self.assertIs(field.serialize(form.null), form.null)
        self.assertEqual(field.serialize(10.34), '10.34')
        self.assertRaises(form.Invalid, field.serialize, u'value')

        self.assertIs(field.deserialize(''), form.null)
        self.assertEqual(field.deserialize('10.34'), 10.34)
        self.assertRaises(form.Invalid, field.deserialize, u'value')

        self.assertEqual(
            strip(field.render(request)),
            '<input type="text" name="test" title="Test" value="10.34" id="test" class="float-widget" />')


class TestDeciamlField(Base):

    def _makeOne(self, name, **kw):
        return form.DecimalField(name, **kw)

    def test_fields_decimal(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', decimal.Decimal('10.34'), {})
        field.update(request)

        self.assertIs(field.serialize(form.null), form.null)
        self.assertEqual(field.serialize(decimal.Decimal('10.34')), '10.34')
        self.assertRaises(form.Invalid, field.serialize, u'value')

        self.assertIs(field.deserialize(''), form.null)
        self.assertEqual(field.deserialize('10.34'), decimal.Decimal('10.34'))
        self.assertRaises(form.Invalid, field.deserialize, u'value')

        self.assertEqual(
            strip(field.render(request)),
            '<input type="text" name="test" title="Test" value="10.34" id="test" class="decimal-widget" />')


class TestLinesField(Base):

    def _makeOne(self, name, **kw):
        return form.LinesField(name, **kw)

    def test_fields_decimal(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', ['1','2','3'], {})
        field.update(request)

        self.assertIs(field.serialize(form.null), form.null)
        self.assertEqual(field.serialize(['1','2','3']), '1\n2\n3')
        self.assertRaises(form.Invalid, field.serialize, 1)

        self.assertIs(field.deserialize(''), form.null)
        self.assertEqual(field.deserialize('1\n2\n3'), ['1','2','3'])
        self.assertRaises(form.Invalid, field.deserialize, 5)

        self.assertEqual(
            strip(field.render(request)),
            '<textarea rows="5" name="test" title="Test" cols="40" id="test" class="textlines-widget">1 2 3</textarea>')


class TestVocabularyField(Base):

    def test_vocabulary_field(self):
        voc = form.SimpleVocabulary.from_items(
            (1, 'one', 'One'),
            (2, 'two', 'Two'),
            (3, 'three', 'Three'))

        self.assertRaises(ValueError, form.VocabularyField, 'test')
        self.assertRaises(
            NotImplementedError,
            form.VocabularyField('test', vocabulary=voc).is_checked,
            voc.get_term(1))

        class MyVocabularyField(form.VocabularyField):
            def is_checked(self, term):
                return term.token == self.form_value

        field = MyVocabularyField('test', vocabulary=voc)
        field.form_value = 'one'

        self.assertTrue(field.is_checked(voc.get_term(1)))
        self.assertFalse(field.is_checked(voc.get_term(2)))

        field.id = 'test'
        field.form_value = form.null
        field.update_items()

        self.assertEqual(field.items,
                         [{'checked': False,
                           'description': None,
                           'id': 'test-0',
                           'label': 'One',
                           'name': 'test',
                           'value': 'one'},
                          {'checked': False,
                           'description': None,
                           'id': 'test-1',
                           'label': 'Two',
                           'name': 'test',
                           'value': 'two'},
                          {'checked': False,
                           'description': None,
                           'id': 'test-2',
                           'label': 'Three',
                           'name': 'test',
                           'value': 'three'}])

        field.form_value = 'one'
        field.update_items()
        self.assertEqual(field.items,
                         [{'checked': True,
                           'description': None,
                           'id': 'test-0',
                           'label': 'One',
                           'name': 'test',
                           'value': 'one'},
                          {'checked': False,
                           'description': None,
                           'id': 'test-1',
                           'label': 'Two',
                           'name': 'test',
                           'value': 'two'},
                          {'checked': False,
                           'description': None,
                           'id': 'test-2',
                           'label': 'Three',
                           'name': 'test',
                           'value': 'three'}])


class TestBaseChoiceField(Base):

    def _makeOne(self, name, **kw):
        return form.BaseChoiceField(name, **kw)

    def test_basechoice(self):
        request = DummyRequest()

        voc = form.SimpleVocabulary.from_items(
            (1, 'one', 'One'),
            (2, 'two', 'Two'),
            (3, 'three', 'Three'))

        field = self._makeOne('test', vocabulary=voc)
        field = field.bind('', 1, {})
        field.update(request)

        self.assertIs(field.extract(), form.null)

        field.params = {'test': '--NOVALUE--'}
        self.assertIs(field.extract(), form.null)

        field.params = {'test': 'three'}
        self.assertIs(field.extract(), 'three')

        self.assertIs(field.serialize(form.null), form.null)
        self.assertEqual(field.serialize(1), 'one')
        self.assertRaises(form.Invalid, field.serialize, 10)
        self.assertRaises(form.Invalid, field.serialize, [1, 10])

        self.assertIs(field.deserialize(''), form.null)
        self.assertEqual(field.deserialize('one'), 1)
        self.assertRaises(form.Invalid, field.deserialize, 5)
        self.assertRaises(form.Invalid, field.deserialize, ['one','five'])


class TestBaseMultiChoiceField(Base):

    def _makeOne(self, name, **kw):
        return form.BaseMultiChoiceField(name, **kw)

    def test_basemultichoice(self):
        request = DummyRequest()

        self.assertRaises(ValueError, self._makeOne, 'test')

        voc = form.SimpleVocabulary.from_items(
            (1, 'one', 'One'),
            (2, 'two', 'Two'),
            (3, 'three', 'Three'))

        orig_field = self._makeOne('test', vocabulary=voc)
        field = orig_field.bind('', [1,3], {})
        field.update(request)

        self.assertIs(field.serialize(form.null), form.null)
        self.assertEqual(field.serialize([1,2]), ['one','two'])
        self.assertRaises(form.Invalid, field.serialize, 1)
        self.assertRaises(form.Invalid, field.serialize, [1, 10])

        self.assertIs(field.deserialize(''), form.null)
        self.assertEqual(field.deserialize(['one','three']), [1,3])
        self.assertRaises(form.Invalid, field.deserialize, 5)
        self.assertRaises(form.Invalid, field.deserialize, ['one','five'])

        field = orig_field.bind('', form.null, {})
        field.update(request)
        self.assertEqual(field.form_value, [])

        field = orig_field.bind('', [1], {})
        field.update(request)
        self.assertEqual(field.form_value, ['one'])

        self.assertIs(field.extract(), form.null)

        field.params = MultiDict((('test', field.noValueToken),
                                  ('test', 'one')))
        self.assertEqual(field.extract(), ['one'])


class TestChoiceField(Base):

    def _makeOne(self, name, **kw):
        return form.ChoiceField(name, **kw)

    def test_vocabulary_field(self):
        request = DummyRequest()

        voc = form.SimpleVocabulary.from_items(
            (1, 'one', 'One'),
            (2, 'two', 'Two'))

        field = self._makeOne('test', vocabulary=voc)
        field = field.bind('', 1, {})
        field.id = 'test'
        field.update(request)

        self.assertEqual(field.items,
                         [{'checked': True,
                           'description': None,
                           'id': 'test-0',
                           'label': 'One',
                           'name': 'test',
                           'value': 'one'},
                          {'checked': False,
                           'description': None,
                           'id': 'test-1',
                           'label': 'Two',
                           'name': 'test',
                           'value': 'two'}])

        field = self._makeOne('test', missing=2, vocabulary=voc)
        field = field.bind('', 1, {})
        field.id = 'test'
        field.update(request)

        self.assertEqual(field.items,
                         [{'checked': False,
                           'description': u'',
                           'id': 'test-novalue',
                           'label': u'select a value ...',
                           'name': 'test',
                           'value': '--NOVALUE--'},
                          {'checked': True,
                           'description': None,
                           'id': 'test-0',
                           'label': 'One',
                           'name': 'test',
                           'value': 'one'},
                          {'checked': False,
                           'description': None,
                           'id': 'test-1',
                           'label': 'Two',
                           'name': 'test',
                           'value': 'two'}])


class TestMultiChoiceField(Base):

    def _makeOne(self, name, **kw):
        return form.MultiChoiceField(name, **kw)

    def test_fields_decimal(self):
        request = DummyRequest()

        voc = form.SimpleVocabulary.from_items(
            (1, 'one', 'One'),
            (2, 'two', 'Two'),
            (3, 'three', 'Three'))

        field = self._makeOne('test', vocabulary=voc)
        field = field.bind('', [1,3], {})
        field.update(request)

        #print strip(field.render(request))


class TestDateTime(unittest.TestCase):
    def _makeOne(self, name='test', *arg, **kw):
        return form.DateTimeField(name, *arg, **kw)

    def _dt(self):
        import datetime
        return datetime.datetime(2010, 4, 26, 10, 48)

    def _today(self):
        import datetime
        return datetime.date.today()

    def test_ctor_default_tzinfo_None(self):
        import iso8601
        typ = self._makeOne()
        self.assertEqual(typ.default_tzinfo.__class__, iso8601.iso8601.Utc)

    def test_ctor_default_tzinfo_non_None(self):
        import iso8601
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        typ = self._makeOne(default_tzinfo=tzinfo)
        self.assertEqual(typ.default_tzinfo, tzinfo)

    def test_serialize_null(self):
        typ = self._makeOne()
        result = typ.serialize(form.null)
        self.assertEqual(result, form.null)

    def test_serialize_with_garbage(self):
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, 'garbage')
        self.assertEqual(e.msg.interpolate(),
                         '"garbage" is not a datetime object')

    def test_serialize_with_date(self):
        import datetime
        typ = self._makeOne()
        date = self._today()
        result = typ.serialize(date)
        expected = datetime.datetime.combine(date, datetime.time())
        expected = expected.replace(tzinfo=typ.default_tzinfo).isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_naive_datetime(self):
        typ = self._makeOne()
        dt = self._dt()
        result = typ.serialize(dt)
        expected = dt.replace(tzinfo=typ.default_tzinfo).isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_none_tzinfo_naive_datetime(self):
        typ = self._makeOne(default_tzinfo=None)
        dt = self._dt()
        result = typ.serialize(dt)
        self.assertEqual(result, dt.isoformat())

    def test_serialize_with_tzware_datetime(self):
        import iso8601
        typ = self._makeOne()
        dt = self._dt()
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        dt = dt.replace(tzinfo=tzinfo)
        result = typ.serialize(dt)
        expected = dt.isoformat()
        self.assertEqual(result, expected)

    def test_deserialize_date(self):
        import datetime
        import iso8601
        date = self._today()
        typ = self._makeOne()
        formatted = date.isoformat()
        result = typ.deserialize(formatted)
        expected = datetime.datetime.combine(result, datetime.time())
        tzinfo = iso8601.iso8601.Utc()
        expected = expected.replace(tzinfo=tzinfo)
        self.assertEqual(result.isoformat(), expected.isoformat())

    def test_deserialize_invalid_ParseError(self):
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, 'garbage')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_null(self):
        typ = self._makeOne()
        result = typ.deserialize(form.null)
        self.assertEqual(result, form.null)

    def test_deserialize_empty(self):
        typ = self._makeOne()
        result = typ.deserialize('')
        self.assertEqual(result, form.null)

    def test_deserialize_success(self):
        import iso8601
        typ = self._makeOne()
        dt = self._dt()
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        dt = dt.replace(tzinfo=tzinfo)
        iso = dt.isoformat()
        result = typ.deserialize(iso)
        self.assertEqual(result.isoformat(), iso)

    def test_deserialize_naive_with_default_tzinfo(self):
        import iso8601
        tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
        typ = self._makeOne(default_tzinfo=tzinfo)
        dt = self._dt()
        dt_with_tz = dt.replace(tzinfo=tzinfo)
        iso = dt.isoformat()
        result = typ.deserialize(iso)
        self.assertEqual(result.isoformat(), dt_with_tz.isoformat())

    def test_deserialize_none_tzinfo(self):
        typ = self._makeOne(default_tzinfo=None)
        dt = self._dt()
        iso = dt.isoformat()
        result = typ.deserialize(iso)
        self.assertEqual(result.isoformat(), dt.isoformat())


class TestDate(unittest.TestCase):
    def _makeOne(self, name='test', *arg, **kw):
        return form.DateField(name, *arg, **kw)

    def _dt(self):
        import datetime
        return datetime.datetime(2010, 4, 26, 10, 48)

    def _today(self):
        import datetime
        return datetime.date.today()

    def test_serialize_null(self):
        val = form.null
        typ = self._makeOne()
        result = typ.serialize(val)
        self.assertEqual(result, form.null)

    def test_serialize_with_garbage(self):
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, 'garbage')
        self.assertEqual(e.msg.interpolate(), '"garbage" is not a date object')

    def test_serialize_with_date(self):
        typ = self._makeOne()
        date = self._today()
        result = typ.serialize(date)
        expected = date.isoformat()
        self.assertEqual(result, expected)

    def test_serialize_with_datetime(self):
        typ = self._makeOne()
        dt = self._dt()
        result = typ.serialize(dt)
        expected = dt.date().isoformat()
        self.assertEqual(result, expected)

    def test_deserialize_invalid_ParseError(self):
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, 'garbage')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_invalid_weird(self):
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, '10-10-10-10')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_null(self):
        typ = self._makeOne()
        result = typ.deserialize(form.null)
        self.assertEqual(result, form.null)

    def test_deserialize_empty(self):
        typ = self._makeOne()
        result = typ.deserialize('')
        self.assertEqual(result, form.null)

    def test_deserialize_success_date(self):
        typ = self._makeOne()
        date = self._today()
        iso = date.isoformat()
        result = typ.deserialize(iso)
        self.assertEqual(result.isoformat(), iso)

    def test_deserialize_success_datetime(self):
        dt = self._dt()
        typ = self._makeOne()
        iso = dt.isoformat()
        result = typ.deserialize(iso)
        self.assertEqual(result.isoformat(), dt.date().isoformat())


class TestFileField(Base):

    def _makeOne(self, name, **kw):
        from ptah.form.fields import FileField
        return FileField(name, **kw)

    def test_fields_text(self):
        request = DummyRequest()

        field = self._makeOne('test')
        field = field.bind('', u'content', {})
        field.update(request)

        self.assertIs(field.extract(), form.null)

        class FileStorage:

            def __init__(self, fp, filename, mt, s):
                self.file = fp
                self.filename = filename
                self.type = mt
                self.length = s

        fs = FileStorage(object(), 'test.jpg', 'image/jpeg', 1024)

        field = field.bind('', u'content', {'test': fs})
        field.update(request)

        res = field.extract()
        self.assertIs(type(res), dict)
        self.assertEqual(res['filename'], 'test.jpg')


class TestJSDateTimeField(Base):

    def _makeOne(self, name, **kw):
        from ptah.form.jsfields import JSDateTimeField
        return JSDateTimeField(name, **kw)

    def test_fields_jsdatetime_update(self):
        from datetime import datetime

        request = DummyRequest()
        field = self._makeOne('test')

        f = field.bind('', form.null, {})
        f.update(request)

        self.assertEqual(f.date_part, u'')
        self.assertEqual(f.time_part, u'')

        dt = datetime(2011, 1, 1, 10, 10)

        f = field.bind('', dt, {})
        f.update(request)

        self.assertEqual(f.date_part, '01/01/2011')
        self.assertEqual(f.time_part, '10:10 AM')

    def test_fields_jsdatetime_extract(self):
        from datetime import datetime

        request = DummyRequest()
        field = self._makeOne('test')

        f = field.bind('', form.null, {})
        f.update(request)
        self.assertIs(f.extract('default'), 'default')

        f = field.bind('', form.null, {'test.date': ''})
        f.update(request)
        self.assertIs(f.extract(), form.null)

        f = field.bind('', form.null,
                       {'test.date': '12/01/2011'})
        f.update(request)
        self.assertIs(f.extract('default'), 'default')

        f = field.bind('', form.null,
                       {'test.date': '12/01/2011',
                        'test.time': ''})
        f.update(request)
        self.assertIs(f.extract(), form.null)

        f = field.bind('', form.null,
                       {'test.date': 'exception',
                        'test.time': 'exception'})
        f.update(request)
        self.assertIs(f.extract(), form.null)

        f = field.bind('', form.null,
                       {'test.date': '12/01/2011',
                        'test.time': '10:10 AM'})
        f.update(request)
        self.assertEqual(f.extract(), '2011-12-01T10:10:00')


class TestJSDateField(Base):

    def _makeOne(self, **kw):
        from ptah.form.jsfields import JSDateField
        return JSDateField('test', **kw)

    def test_serialize_null(self):
        typ = self._makeOne()
        result = typ.serialize(form.null)
        self.assertEqual(result, form.null)

    def test_serialize_with_garbage(self):
        typ = self._makeOne()
        e = invalid_exc(typ.serialize, 'garbage')
        self.assertEqual(e.msg.interpolate(),
                         '"garbage" is not a date object')

    def test_serialize_with_date(self):
        import datetime
        typ = self._makeOne()
        date = datetime.date.today()
        result = typ.serialize(date)
        expected = date.strftime('%m/%d/%Y')
        self.assertEqual(result, expected)

    def test_serialize_with_datetime(self):
        import datetime
        typ = self._makeOne()
        dt = datetime.datetime.now()
        result = typ.serialize(dt)
        expected = dt.strftime('%m/%d/%Y')
        self.assertEqual(result, expected)

    def test_deserialize_invalid(self):
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, 'garbage')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_invalid_weird(self):
        typ = self._makeOne()
        e = invalid_exc(typ.deserialize, '10-10-10-10')
        self.failUnless('Invalid' in e.msg)

    def test_deserialize_null(self):
        typ = self._makeOne()
        result = typ.deserialize(form.null)
        self.assertEqual(result, form.null)

    def test_deserialize_empty(self):
        typ = self._makeOne()
        result = typ.deserialize('')
        self.assertEqual(result, form.null)

    def test_deserialize_success_date(self):
        import datetime
        typ = self._makeOne()
        date = datetime.date.today()
        iso = date.strftime('%m/%d/%Y')
        result = typ.deserialize(iso)
        self.assertEqual(result, date)

    def test_deserialize_success_datetime(self):
        import datetime

        dt = datetime.datetime.now()
        typ = self._makeOne()
        iso = dt.strftime('%m/%d/%Y')
        result = typ.deserialize(iso)
        self.assertEqual(result.isoformat(), dt.date().isoformat())



""" ptah settings """
import colander
import translationstring
from ptah import config

_ = translationstring.TranslationStringFactory('ptah')


SECURITY = config.register_settings(
    'auth',

    config.SchemaNode(
        colander.Bool(),
        name = 'policy',
        title = _('Authentication policy'),
        description = _('Enable authentication policy.'),
        required = False,
        default = False),

    config.SchemaNode(
        colander.Str(),
        name = 'secret',
        title = _('Policy secret'),
        description = _('The secret (a string) used for auth_tkt '
                        'cookie signing'),
        required = False),

    config.SchemaNode(
        colander.Bool(),
        name = 'authorization',
        title = _('Authorization policy'),
        description = _('Enable/disable default authorization policy.'),
        required = False,
        default = True),

    title = _('Pyramid authentication settings'),
)


SESSION = config.register_settings(
    'session',

    config.SchemaNode(
        colander.Str(),
        name = 'type',
        title = _('The name of the back-end'),
        description = _('Available back-ends supplied with Beaker: file, dbm, '
                        'memory, ext:memcached, ext:database, ext:google'),
        default = ''),

    config.SchemaNode(
        colander.Str(),
        name = 'data_dir',
        title = _('Data directory'),
        description = _('Used with any back-end that stores its data in '
                        'physical files, such as the dbm or file-based '
                        'back-ends. This path should be an absolute path '
                        'to the directory that stores the files.'),
        default = ''),

    config.SchemaNode(
        colander.Str(),
        name = 'lock_dir',
        title = _('Lock directory'),
        description = _('Used with every back-end, to coordinate locking. '
                        'With caching, this lock file is used to ensure that '
                        "multiple processes/threads aren't attempting to "
                        're-create the same value at the same '
                        'time (The Dog-Pile Effect)'),
        default = ''),

    config.SchemaNode(
        colander.Str(),
        name = 'url',
        title = _('URL'),
        description = _('URL is specific to use of either ext:memcached or '
                        'ext:database. When using one of those types, '
                        'this option is required.'),
        default = ''),

    config.SchemaNode(
        colander.Str(),
        name = 'key',
        required = False,
        title = _('Key'),
        default = '',
        description = _('Name of the cookie key used to save the '
                        'session under.')),

    config.SchemaNode(
        colander.Str(),
        name = 'secret',
        required = False,
        title = _('Secret'),
        default = '',
        description = _('Used with the HMAC to ensure session integrity. '
                        'This value should ideally be a randomly '
                        'generated string.')),

    title = _('Pyramid session'),
    description = _('Beaker session configuration settings'),
    validator = (config.RequiredWithDependency('key', 'type', default=''),
                 config.RequiredWithDependency('secret', 'type', default='')),
)

SQLA = config.register_settings(
    'sqla',

    config.SchemaNode(
        colander.Str(),
        name = 'url',
        default = '',
        title = 'Engine URL',
        description = 'SQLAlchemy database engine URL'),

    config.SchemaNode(
        colander.Bool(),
        name = 'cache',
        default = True,
        title = 'Cache',
        description = 'Eanble SQLAlchemy statement caching'),

    title = 'SQLAlchemy settings',
    description = 'Configuration settings for a SQLAlchemy database engine.'
    )


MAIL = config.register_settings(
    'mail',

    config.SchemaNode(
        colander.Str(),
        name = 'host',
        title = 'Host',
        description = 'SMTP Server host name.',
        default = 'localhost'),

    config.SchemaNode(
        colander.Int(),
        name = 'port',
        title = 'Port',
        description = 'SMTP Server port number.',
        default = 25),

    config.SchemaNode(
        colander.Str(),
        name = 'username',
        title = 'Username',
        description = 'SMTP Auth username.',
        default = ''),

    config.SchemaNode(
        colander.Str(),
        name = 'password',
        title = 'Password',
        description = 'SMTP Auth password.',
        default = ''),

    config.SchemaNode(
        colander.Bool(),
        name = 'no_tls',
        title = 'No tls',
        description = 'Disable TLS.',
        default = False),

    config.SchemaNode(
        colander.Bool(),
        name = 'force_tls',
        title = 'Force TLS',
        description = 'Force use TLS.',
        default = False),

    config.SchemaNode(
        colander.Bool(),
        name = 'debug',
        title = 'Debug',
        description = 'Debug smtp.',
        default = False),

    config.SchemaNode(
        colander.Str(),
        name = 'from_name',
        default = 'Site administrator'),

    config.SchemaNode(
        colander.Str(),
        name = 'from_address',
        validator = colander.Email(),
        required = False,
        default = 'administrator@localhost.org'),

    title = 'Mail settings',
    description = 'Configuration settings for application mail.',
)



""" content helper forms """
import re
from ptah import view, form
from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPFound

from security import wrap
from interfaces import ContentNameSchema


class AddForm(form.Form):

    tinfo = None
    container = None

    name_show = True
    name_suffix = ''
    name_widgets = None
    name_fields = ContentNameSchema

    def __init__(self, context, request):
        self.container = context
        super(AddForm, self).__init__(context, request)

    @reify
    def fields(self):
        return self.tinfo.fieldset

    @reify
    def label(self):
        return 'Add %s'%self.tinfo.title

    @reify
    def description(self):
        return self.tinfo.description

    def chooseName(self, **kw):
        name = kw.get('title', u'')

        name = re.sub(
            r'-{2,}', '-',
            re.sub('^\w-|-\w-|-\w$', '-',
                   re.sub(r'\W', '-', name.strip()))).strip('-').lower()

        suffix = self.name_suffix
        n = '%s%s'%(name, suffix)
        i = 0
        while n in self.container:
            i += 1
            n = u'%s-%s%s'%(name, i, suffix)

        return n.replace('/', '-').lstrip('+@')

    def update(self):
        self.name_suffix = getattr(self.tinfo, 'name_suffix', '')

        self.tinfo.check_context(self.container)
        super(AddForm, self).update()

    def update_widgets(self):
        super(AddForm, self).update_widgets()

        if self.name_show:
            self.name_widgets = \
                   form.FormWidgets(self.name_fields, self, self.request)
            self.name_widgets.mode = self.mode
            self.name_widgets.update()

    def validate(self, data, errors):
        super(AddForm, self).validate(data, errors)

        if self.name_widgets and '__name__' in data and data['__name__']:
            name = data['__name__']
            if name in self.container.keys():
                error = form.Invalid(
                    self.name_widgets['__name__'], 'Name already in use')
                error.field = self.name_widgets['__name__']
                errors.append(error)

    def extract(self):
        data, errors = self.widgets.extract()

        if self.name_show:
            name_data, name_errors = self.name_widgets.extract()
            if name_errors:
                errors.extend(name_errors)

            data.update(name_data)

        return data, errors

    def create(self, **data):
        name = data.get('__name__')
        if not name:
            name = self.chooseName(**data)

        return wrap(self.container).create(
            self.tinfo.__uri__, name, **data)

    @form.button('Add', actype=form.AC_PRIMARY)
    def add_handler(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        content = self.create(**data)

        self.message('New content has been created.')
        raise HTTPFound(location=self.get_next_url(content))

    @form.button('Cancel')
    def cancel_handler(self):
        raise HTTPFound(location='.')

    def get_next_url(self, content):
        return self.request.resource_url(content)


view.register_snippet(
    'form-actions', AddForm,
    template = view.template('ptah.cms:form-actions.pt'))


class EditForm(form.Form):

    @reify
    def label(self):
        return 'Modify content: %s'%self.tinfo.title

    @reify
    def fields(self):
        return self.tinfo.fieldset

    def form_content(self):
        data = {}
        for name, field in self.tinfo.fieldset.items():
            data[name] = getattr(self.context, name, field.default)

        return data

    def update(self):
        self.tinfo = self.context.__type__

        super(EditForm, self).update()

    def apply_changes(self, **data):
        wrap(self.context).update(**data)

    @form.button('Save', actype=form.AC_PRIMARY)
    def save_handler(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        self.apply_changes(**data)

        self.message("Changes have been saved.")
        raise HTTPFound(location=self.get_next_url())

    @form.button('Cancel')
    def cancel_handler(self):
        raise HTTPFound(location=self.get_next_url())

    def get_next_url(self):
        return '.'



import ptah
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound, HTTPForbidden

from base import Base


class Content(ptah.cms.Content):

    __type__ = ptah.cms.Type('content', 'Test Content',
                             description = 'Test Content description')
    __uri_factory__ = ptah.UriFactory('cms-content')


class Container(ptah.cms.Container):

    __type__ = ptah.cms.Type('container', 'Test Container')
    __uri_factory__ = ptah.UriFactory('cms-container')


class TestAddForm(Base):

    def test_addform_ctor(self):
        from ptah.cms.forms import AddForm

        container = Container()
        form = AddForm(container, DummyRequest())
        self.assertIs(form.container, container)

    def test_addform_basics(self):
        from ptah.cms.forms import AddForm

        container = Container()

        form = AddForm(container, DummyRequest())
        form.tinfo = Content.__type__

        self.assertIs(form.fields, Content.__type__.fieldset)
        self.assertEqual(form.label, 'Add %s'%Content.__type__.title)
        self.assertEqual(form.description, Content.__type__.description)

    def test_addform_choosename(self):
        from ptah.cms.forms import AddForm

        container = Container()

        form = AddForm(container, DummyRequest())

        name = form.chooseName(title='Test title')
        self.assertEqual(name, 'test-title')

    def test_addform_choosename_existing(self):
        from ptah.cms.forms import AddForm

        container = Container()
        container['test-title'] = Content()

        form = AddForm(container, DummyRequest())

        name = form.chooseName(title='Test title')
        self.assertEqual(name, 'test-title-1')

    def test_addform_choosename_suffix(self):
        from ptah.cms.forms import AddForm

        container = Container()

        form = AddForm(container, DummyRequest())
        form.name_suffix = '.html'

        name = form.chooseName(title='Test title')
        self.assertEqual(name, 'test-title.html')

    def test_addform_update_suffix_from_type(self):
        from ptah.cms.forms import AddForm

        container = Container()

        form = AddForm(container, DummyRequest())

        Content.__type__.name_suffix = '.xml'
        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED

        form.tinfo = Content.__type__
        form.update()

        self.assertEqual(form.name_suffix, '.xml')

        Content.__type__.name_suffix = ''
        Content.__type__.permission = ptah.cms.NOT_ALLOWED

    def test_addform_update_type_check_context(self):
        from ptah.cms.forms import AddForm

        container = Container()

        form = AddForm(container, DummyRequest())

        Content.__type__.permission = ptah.cms.NOT_ALLOWED
        form.tinfo = Content.__type__

        self.assertRaises(HTTPForbidden, form.update)

    def test_addform_name_widgets(self):
        from ptah.cms.forms import AddForm

        form = AddForm(Container(), DummyRequest())

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        self.assertIsNotNone(form.name_widgets)
        self.assertIn('__name__', form.name_widgets)

    def test_addform_no_name_widgets(self):
        from ptah.cms.forms import AddForm

        form = AddForm(Container(), DummyRequest())

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.name_show = False
        form.update()

        self.assertIsNone(form.name_widgets)

    def test_addform_validate_name(self):
        from ptah.cms.forms import AddForm

        container = Container()
        container['test'] = Content()

        form = AddForm(container, DummyRequest())

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        errors = []
        form.validate({'__name__': 'test'}, errors)

        self.assertEqual(len(errors), 1)
        self.assertIs(errors[0].field, form.name_widgets['__name__'])
        self.assertEqual(errors[0].msg, 'Name already in use')

        errors = []
        form.validate({'__name__': 'normal-name'}, errors)
        self.assertEqual(len(errors), 0)

    def test_addform_extract_with_errors(self):
        from ptah.cms.forms import AddForm

        form = AddForm(Container(), DummyRequest(
            POST={'__name__': 't/est-content'}))

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        data, errors = form.extract()

        self.assertEqual(len(errors), 2)
        self.assertEqual(errors[0].field.name, 'title')
        self.assertEqual(errors[0].msg, 'Required')
        self.assertEqual(errors[1].field.name, '__name__')
        self.assertEqual(errors[1].msg, "Names cannot contain '/'")

    def test_addform_extract_with_errors_no_name(self):
        from ptah.cms.forms import AddForm

        form = AddForm(Container(), DummyRequest(
            POST={'__name__': 't/est-content'}))

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.name_show = False
        form.update()

        data, errors = form.extract()

        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0].field.name, 'title')
        self.assertEqual(errors[0].msg, 'Required')

    def test_addform_extract(self):
        from ptah.cms.forms import AddForm

        form = AddForm(Container(), DummyRequest(
            POST={'title': 'Test Content',
                  '__name__': 'test-content'}))

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        data, errors = form.extract()

        self.assertEqual(len(errors), 0)
        self.assertEqual(len(data), 3)
        self.assertIn('title', data)
        self.assertIn('description', data)
        self.assertIn('__name__', data)

    def test_addform_extract_no_name(self):
        from ptah.cms.forms import AddForm

        form = AddForm(Container(), DummyRequest(
            POST={'title': 'Test Content',
                  '__name__': 'test-content'}))

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.name_show = False
        form.update()

        data, errors = form.extract()

        self.assertEqual(len(errors), 0)
        self.assertEqual(len(data), 2)
        self.assertIn('title', data)
        self.assertIn('description', data)

    def test_addform_create_empty_name(self):
        from ptah.cms.forms import AddForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        form = AddForm(Container(), DummyRequest())

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        content = form.create(**{'title': 'Test Content'})

        self.assertEqual(content.__name__, 'test-content')
        self.assertEqual(content.title, 'Test Content')
        self.assertIsInstance(content, Content)

    def test_addform_create(self):
        from ptah.cms.forms import AddForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        form = AddForm(Container(), DummyRequest())

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        content = form.create(**{'title': 'Test Content',
                                 '__name__': 'page.html'})

        self.assertEqual(content.__name__, 'page.html')
        self.assertIsInstance(content, Content)

    def test_addform_add(self):
        from ptah.cms.forms import AddForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        container = Container()
        request = DummyRequest(
            POST = {'title': 'Test Content',
                    'form.buttons.add': 'Add'})
        request.root = container
        request.root.__path__ = '/'
        request.root.__root_path__ = '/'

        form = AddForm(container, request)

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__

        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '/test-content/')
        self.assertIn('New content has been created.',
                      request.session['msgservice'][0])

    def test_addform_add_errors(self):
        from ptah.cms.forms import AddForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        container = Container()
        request = DummyRequest(
            POST = {'form.buttons.add': 'Add'})

        form = AddForm(container, request)
        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__
        form.update()

        self.assertIn('Please fix indicated errors.',
                      request.session['msgservice'][0])

    def test_addform_cancel(self):
        from ptah.cms.forms import AddForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        container = Container()
        request = DummyRequest(
            POST = {'form.buttons.cancel': 'Cancel'})

        form = AddForm(container, request)

        Content.__type__.permission = ptah.cms.NO_PERMISSION_REQUIRED
        form.tinfo = Content.__type__

        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')


class TestEditForm(Base):

    def test_editform_basics(self):
        from ptah.cms.forms import EditForm

        content = Content()

        form = EditForm(content, DummyRequest())
        form.update()

        self.assertIs(form.fields, Content.__type__.fieldset)
        self.assertIs(form.tinfo, Content.__type__)
        self.assertEqual(form.label,'Modify content: %s'%Content.__type__.title)

    def test_editform_form_content(self):
        from ptah.cms.forms import EditForm

        content = Content()
        content.title = 'Test content'
        content.description = 'Desc'

        form = EditForm(content, DummyRequest())
        form.update()

        data = form.form_content()

        self.assertEqual(data['title'], 'Test content')
        self.assertEqual(data['description'], 'Desc')

    def test_editform_apply_changes(self):
        from ptah.cms.forms import EditForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        content = Content()
        content.title = 'Test'
        content.description = 'Desc'

        form = EditForm(content, DummyRequest())
        form.update()

        form.apply_changes(**{'title': 'Test2', 'description': 'Desc2'})

        self.assertEqual(content.title, 'Test2')
        self.assertEqual(content.description, 'Desc2')

    def test_editform_save(self):
        from ptah.cms.forms import EditForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        content = Content()
        content.title = 'Test'
        content.description = 'Desc'

        form = EditForm(content, DummyRequest(
            POST = {'title': 'Test2', 'description': 'Desc2',
                    'form.buttons.save': 'Save'}))

        try:
            form.update()
        except Exception, res:
            pass

        self.assertEqual(res.headers['location'], '.')
        self.assertEqual(content.title, 'Test2')
        self.assertEqual(content.description, 'Desc2')
        self.assertIn('Changes have been saved.',
                      form.request.session['msgservice'][0])

    def test_editform_save_errors(self):
        from ptah.cms.forms import EditForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        content = Content()

        form = EditForm(content, DummyRequest(
            POST = {'form.buttons.save': 'Save'}))

        form.update()
        self.assertIn('Please fix indicated errors.',
                      form.request.session['msgservice'][0])

    def test_editform_cancel(self):
        from ptah.cms.forms import EditForm
        ptah.authService.set_userid(ptah.SUPERUSER_URI)

        content = Content()

        form = EditForm(content, DummyRequest(
            POST = {'form.buttons.cancel': 'Cancel'}))

        try:
            form.update()
        except Exception, res:
            pass

        self.assertEqual(res.headers['location'], '.')



import logging
import tempfile
import threading
import pkg_resources
from docutils import io
from docutils.core import Publisher

from sphinx.application import Sphinx
from sphinx.writers.html import HTMLWriter, HTMLTranslator

log = logging.getLogger('ptah.rst')

local_data = threading.local()

confdir = '%s/docs/' % pkg_resources.get_distribution('ptah').location
outdir = tempfile.mkdtemp()
srcdir = tempfile.mkdtemp()
doctreedir = tempfile.mkdtemp()


def get_sphinx():
    sphinx = getattr(local_data, 'sphinx', None)
    if sphinx is None:
        sphinx = Sphinx(srcdir, confdir, outdir, doctreedir, 'pickle')
        sphinx.builder.translator_class = CustomHTMLTranslator

        sphinx.env.patch_lookup_functions()
        sphinx.env.temp_data['docname'] = 'text'
        sphinx.env.temp_data['default_domain'] = 'py'

        pub = Publisher(reader=None,
                        parser=None,
                        writer=HTMLWriter(sphinx.builder),
                        source_class=io.StringInput,
                        destination_class=io.NullOutput)
        pub.set_components('standalone', 'restructuredtext', None)
        pub.process_programmatic_settings(None, sphinx.env.settings, None)
        pub.set_destination(None, None)

        sphinx.publisher = pub

        local_data.sphinx = sphinx

    return sphinx, sphinx.publisher


def rst_to_html(text):
    if not isinstance(text, unicode):
        text = unicode(text, 'utf-8')

    sphinx, pub = get_sphinx()

    pub.set_source(text, None)

    try:
        pub.publish()
    except:
        log.warning(u'ReST to HTML error\n %s', text)
        return u'<pre>%s</pre>' % text

    doctree = pub.document
    sphinx.env.filter_messages(doctree)
    for domain in sphinx.env.domains.itervalues():
        domain.process_doc(sphinx.env, 'text', doctree)

    pub.writer.write(doctree, io.StringOutput(encoding='unicode'))
    pub.writer.assemble_parts()

    parts = pub.writer.parts
    return u''.join((parts['body_pre_docinfo'],
                     parts['docinfo'], parts['body']))


class CustomHTMLTranslator(HTMLTranslator):

    def visit_pending_xref(self, node):
        pass

    def depart_pending_xref(self, node):
        pass



""" This is an example of layouts """
from paste.httpserver import serve
from ptah import view, cms


class PageLayout(view.Layout):
    view.layout('page', cms.ApplicationRoot,
                layer = 'test',
                template = view.template('templates/layoutpage.pt'))

    """ override 'page' layout from ptah.cmsapp

    layer - identifier, import order does matter, last imported wins
    """

    def render(self, content, **kwargs):
        """ default implementation, just example. in most cases
        default implementation is ok. """
        if self.template is None:
            return content

        kwargs.update({'view': self,
                       'content': content,
                       'context': self.context,
                       'request': self.request,
                       'format': format})

        return self.template(**kwargs)


class WorkspaceLayout(view.Layout):
    view.layout('workspace', cms.ApplicationRoot,
                parent = 'page',
                layer = 'test',
                template = view.template('templates/layoutworkspace.pt'))

    """ same as PageLayout, it uses 'page' as parent layout """

    def update(self):
        self.user = ptah.authService.get_current_principal()
        self.isAnon = self.user is None


if __name__ == '__main__':
    """

    """
    import ptah
    app = ptah.make_wsgi_app({'settings':r'./ptah.ini'})
    serve(app, host='0.0.0.0')



import ptah
import transaction

from base import Base


class Content(ptah.cms.Content):

    __type__ = ptah.cms.Type('content', 'Test Content')
    __uri_factory__ = ptah.UriFactory('cms-content')


class Container(ptah.cms.Container):

    __type__ = ptah.cms.Type('container', 'Test Container')
    __uri_factory__ = ptah.UriFactory('cms-container')


class TestContainer(Base):

    def test_container_basics(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        self.assertEqual(content.__path__, '')

        self.assertRaises(
            ValueError, container.__setitem__, 'content', object())

        self.assertRaises(
            ValueError, container.__setitem__, 'content', container)

        container['content'] = content
        self.assertEqual(content.__name__, 'content')
        self.assertEqual(content.__path__, '/container/content/')
        self.assertEqual(content.__parent__, container)
        self.assertEqual(content.__parent_uri__, container.__uri__)

        content_uri = content.__uri__
        container_uri = container.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        self.assertTrue('content' in container)
        self.assertEqual(container.keys(), ['content'])
        self.assertEqual(container['content'].__uri__, content_uri)
        self.assertEqual(container.get('content').__uri__, content_uri)
        self.assertEqual(container.get('unknown'), None)
        self.assertRaises(KeyError, container.__getitem__, 'unknown')

        content = container['content']
        self.assertRaises(
            KeyError, container.__setitem__, 'content', content)

    def test_container_keys_values_del(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content = Content(title='Content')

        container['content'] = content
        self.assertEqual(container.keys(), ['content'])
        self.assertEqual(container.values(), [content])

        del container['content']
        self.assertEqual(container.keys(), [])
        self.assertEqual(container.values(), [])

    def test_container_keys_with_local(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content = Content(title='Content')
        container['content'] = content

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        content_uri = content.__uri__
        container_uri = container.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)

        c2 = Content(title='Content2')
        container['content2'] = c2

        self.assertEqual(container.keys(), ['content', 'content2'])
        self.assertEqual([c.__uri__ for c in container.values()],
                         [content_uri, c2.__uri__])

        del container['content']

        self.assertEqual(container.keys(), ['content2'])
        self.assertEqual([c.__uri__ for c in container.values()],
                         [c2.__uri__])

    def test_container_simple_move(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        container['content'] = content

        content_uri = content.__uri__
        container_uri = container.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        content = ptah.resolve(content_uri)

        container['moved'] = content

        self.assertEqual(content.__name__, 'moved')
        self.assertEqual(content.__path__, '/container/moved/')
        self.assertEqual(content.__parent__, container)
        self.assertEqual(content.__parent_uri__, container.__uri__)
        transaction.commit()

        container = ptah.resolve(container_uri)
        self.assertEqual(container.keys(), ['moved'])
        self.assertEqual(container['moved'].__uri__, content_uri)

    def test_container_getitem(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content1 = Content(title='Content1')
        content2 = Content(title='Content2')

        container['content1'] = content1
        container['content2'] = content2

        self.assertEqual(container['content1'].__uri__, content1.__uri__)
        self.assertEqual(container['content2'].__uri__, content2.__uri__)
        self.assertEqual(container.get('content1').__uri__, content1.__uri__)
        self.assertEqual(container.get('content2').__uri__, content2.__uri__)

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(content1)
        ptah.cms.Session.add(content2)
        ptah.cms.Session.flush()

        c_u = container.__uri__
        c1_u = content1.__uri__
        c2_u = content2.__uri__
        transaction.commit()

        container = ptah.resolve(c_u)
        self.assertEqual(container['content1'].__uri__, c1_u)
        self.assertEqual(container['content2'].__uri__, c2_u)
        transaction.commit()

        container = ptah.resolve(c_u)
        self.assertEqual(container.get('content1').__uri__, c1_u)
        self.assertEqual(container.get('content2').__uri__, c2_u)

    def test_container_items(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content1 = Content(title='Content1')
        content2 = Content(title='Content2')

        container['content1'] = content1
        container['content2'] = content2

        self.assertEqual(list(container.items()),
                         [('content1', content1), ('content2', content2)])

    def test_container_simple_move_to_subtree(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        folder = Container(title='Folder')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(folder)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        container['content'] = content
        container['folder'] = folder

        content_uri = content.__uri__
        container_uri = container.__uri__
        folder_uri = folder.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        content = ptah.resolve(content_uri)
        folder = ptah.resolve(folder_uri)

        self.assertEqual(container.keys(), ['folder', 'content'])

        folder['new-content'] = content

        self.assertEqual(content.__name__, 'new-content')
        self.assertEqual(content.__path__, '/container/folder/new-content/')
        self.assertEqual(content.__parent__, folder)
        self.assertEqual(content.__parent_uri__, folder.__uri__)
        transaction.commit()

        folder = ptah.resolve(folder_uri)
        container = ptah.resolve(container_uri)
        self.assertEqual(container.keys(), ['folder'])
        self.assertEqual(folder.keys(), ['new-content'])

    def test_container_insert_subtree(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        folder = Container(title='Folder')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(folder)
        ptah.cms.Session.add(content)

        folder['content'] = content
        container['folder'] = folder

        content_uri = content.__uri__
        folder_uri = folder.__uri__
        container_uri = container.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        content = ptah.resolve(content_uri)
        folder = ptah.resolve(folder_uri)

        self.assertEqual(container.keys(), ['folder'])
        self.assertEqual(folder.keys(), ['content'])
        self.assertEqual(folder.__path__, '/container/folder/')
        self.assertEqual(content.__path__, '/container/folder/content/')
        transaction.commit()

    def test_container_simple_rename_subtree(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        folder1 = Container(title='Folder1')
        folder2 = Container(title='Folder2')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(folder1)
        ptah.cms.Session.add(folder2)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        container['folder1'] = folder1
        folder1['folder2'] = folder2
        folder2['content'] = content

        content_uri = content.__uri__
        container_uri = container.__uri__
        folder1_uri = folder1.__uri__
        folder2_uri = folder2.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)

        container['new-folder'] = container['folder1']
        transaction.commit()

        folder1 = ptah.resolve(folder1_uri)
        folder2 = ptah.resolve(folder2_uri)
        content = ptah.resolve(content_uri)
        container = ptah.resolve(container_uri)

        self.assertEqual(container.keys(), ['new-folder'])
        self.assertEqual(folder1.__path__, '/container/new-folder/')
        self.assertEqual(folder2.__path__, '/container/new-folder/folder2/')
        self.assertEqual(content.__path__,
                         '/container/new-folder/folder2/content/')

    def test_container_move_self_recursevly(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        folder = Container(title='Folder')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(folder)
        ptah.cms.Session.flush()

        container['folder'] = folder

        folder_uri = folder.__uri__
        container_uri = container.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        folder = ptah.resolve(folder_uri)

        self.assertRaises(
            TypeError, folder.__setitem__, 'subfolder', container)

    def test_container_delete(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        container['content'] = content

        content_uri = content.__uri__
        container_uri = container.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        del container['content']
        self.assertEqual(container.keys(), [])
        self.assertEqual(container.values(), [])

        transaction.commit()

        self.assertTrue(ptah.resolve(content_uri) is None)

        container = ptah.resolve(container_uri)
        self.assertEqual(container.keys(), [])
        self.assertRaises(KeyError, container.__delitem__, 'content')
        self.assertRaises(KeyError, container.__delitem__, Content())

    def test_container_delete_recursive(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        folder = Container(title='Folder')
        content = Content(title='Content')

        ptah.cms.Session.add(container)
        ptah.cms.Session.add(folder)
        ptah.cms.Session.add(content)
        ptah.cms.Session.flush()

        container['folder'] = folder
        folder['content'] = content

        content_uri = content.__uri__
        container_uri = container.__uri__
        folder_uri = folder.__uri__
        transaction.commit()

        container = ptah.resolve(container_uri)
        del container['folder']
        transaction.commit()

        self.assertTrue(ptah.resolve(content_uri) is None)
        self.assertTrue(ptah.resolve(folder_uri) is None)

    def test_container_setitem_parent_not_node(self):
        container = Container(__name__ = 'container', __path__ = '/container/')
        content = Content(title='Content')

        container.__parent__ = object()

        # this line should not fail
        container['content'] = content

        self.assertIs(container['content'], content)

    def test_container_create(self):
        container = Container(__name__ = 'container', __path__ = '/container/')

        self.assertRaises(
            ptah.cms.NotFound, container.create, 'unknown', 'test')

        tinfo = Content.__type__
        tname = tinfo.__uri__

        self.assertRaises(
            ptah.cms.Forbidden, container.create, tname, 'test')

        tinfo.permission = None

        self.assertRaises(
            ptah.cms.Error, container.create, tname, 'te/st')

        self.assertRaises(
            ptah.cms.Error, container.create, tname, ' test')

        content = container.create(tname, 'test', title = 'Test title')
        self.assertIsInstance(content, Content)
        self.assertIsNotNone(content.created)
        self.assertIsNotNone(content.modified)
        self.assertEqual(content.title, 'Test title')
        self.assertEqual(container.keys(), ['test'])

        self.assertRaises(
            ptah.cms.Error, container.create,
            tname, 'test', title = 'Test title')

        tinfo.permission = ptah.NOT_ALLOWED

    def test_container_info(self):
        container = Container(__name__ = 'container', __path__ = '/container/')

        info = container.info()
        self.assertEqual(info['__name__'], 'container')
        self.assertTrue(info['__container__'])
